"""技能自动选择器

基于 SkillMeta 的 `triggers`、`name`、`summary` 进行简单匹配，选择最合适的技能。
规则：
- 仅考虑 `enabled=True` 的技能
- 主要依据 `triggers`（大小写不敏感的子串匹配），每命中一个触发词+2分
- 次要依据 `name` 与 `summary`（每命中+1分）
- 返回分数最高且分数>0 的技能
"""
from __future__ import annotations

from typing import List, Optional, Any, Dict
import json
import logging
import re

logger = logging.getLogger("ketacli.textual.ai_helpers")

def _safe_json_loads(text: str) -> Dict:
    """宽容解析模型返回的JSON，自动剥离```json代码块与前后噪音。
    返回字典；失败则返回空字典。
    """
    if not text:
        return {}
    s = text.strip()
    # 去掉```json/``` 包裹
    if s.startswith("```"):
        try:
            s = s.strip().strip("`")
            # 去掉可能的语言标记，如json
            if s.lower().startswith("json"):
                s = s[4:].strip()
        except Exception:
            pass
    # 粗略截取第一个{到最后一个}
    try:
        l = s.find("{")
        r = s.rfind("}")
        if l != -1 and r != -1 and r > l:
            s = s[l:r+1]
    except Exception:
        pass
    try:
        return json.loads(s)
    except Exception:
        return {}

from .models import SkillMeta


def _norm(text: str) -> str:
    return (text or "").strip().lower()

# 轻量停用词（中英文混合），用于过滤通用性极强、易引发误命中的词
STOPWORDS = {
    # 中文常用泛词
    "查询", "搜索", "查找", "日志", "指标", "数据", "结果", "请问", "帮我", "一下", "怎么", "如何", "查看", "展示", "显示", "统计", "分析", "情况", "信息", "内容",
    # 英文常见泛词
    "search", "query", "log", "logs", "metric", "metrics", "data", "result", "results", "show", "display", "view", "find", "help", "please", "how", "check", "list",
}

def _tokens(text: str) -> List[str]:
    """粗粒度分词：
    - 英文/数字：使用正则提取连续的字母数字下划线
    - 中文：保留原字符串整体作为token（不引入分词依赖）
    最终用于强命中的精确匹配；停用词会在后续过滤。
    """
    s = _norm(text)
    ascii_tokens = re.findall(r"[a-z0-9_]+", s)
    # 对于包含空白的情况，额外按空格切分（兼容混合输入）
    space_tokens = [t for t in s.split() if t]
    # 合并并去重
    tokens = list({*ascii_tokens, *space_tokens})
    return tokens


def score_skill(user_text: str, meta: SkillMeta) -> tuple[int, dict]:
    """
    计算技能与用户输入的匹配分，并返回细节：
    - 触发词子串命中 +2/次
    - 名称子串命中 +1
    - 摘要子串命中 +1
    """
    text = _norm(user_text)
    name_n = _norm(meta.name)
    summ_n = _norm(meta.summary or "")

    # 分词并过滤停用词（仅影响强命中）
    tokens = [t for t in _tokens(text) if t and t not in STOPWORDS]
    token_set = set(tokens)

    strong_hits = 0
    weak_hits = 0
    for t in (meta.triggers or []):
        tt = _norm(t)
        if not tt:
            continue
        # 强命中：精确token命中，或中文/长词(>=3)的子串命中
        if tt in token_set or (len(tt) >= 3 and tt in text):
            strong_hits += 1
        # 弱命中：其余子串命中（避免短词造成过度匹配）
        elif tt and tt in text:
            weak_hits += 1

    name_hit = 1 if name_n and name_n in text else 0
    summary_hit = 1 if summ_n and summ_n in text else 0
    # 权重：强命中+3，弱命中+1，名称/摘要各+1
    score = strong_hits * 3 + weak_hits * 1 + name_hit + summary_hit
    detail = {
        "score": score,
        "strong_hits": strong_hits,
        "weak_hits": weak_hits,
        "name_hit": bool(name_hit),
        "summary_hit": bool(summary_hit),
        "tokens": tokens,
    }
    return score, detail


def select_best_skill(user_text: str, metas: List[SkillMeta]) -> Optional[SkillMeta]:
    text = _norm(user_text)
    if not text:
        return None
    best: Optional[SkillMeta] = None
    best_score: int = 0
    for meta in metas or []:
        if not getattr(meta, "enabled", True):
            continue
        score, _ = score_skill(text, meta)
        if score > best_score:
            best = meta
            best_score = score
    return best if best_score > 0 else None

def select_best_skill_with_explain(user_text: str, metas: List[SkillMeta]) -> Dict:
    """返回最佳技能及评分细节，便于调试或日志记录。
    结构：{"best": SkillMeta|None, "detail": {"score": int, ...}}
    注意：使用"|"语法需要from __future__ import annotations
    """
    text = _norm(user_text)
    if not text:
        return {"best": None, "detail": {"score": 0}}
    best: Optional[SkillMeta] = None
    best_detail: Dict = {"score": 0}
    for meta in metas or []:
        if not getattr(meta, "enabled", True):
            continue
        score, detail = score_skill(text, meta)
        if score > best_detail.get("score", 0):
            best = meta
            best_detail = {**detail, "name": meta.name}
    return {"best": best if (best_detail.get("score", 0) > 0) else None, "detail": best_detail}


async def select_skills_by_model(ai_client: Any, user_text: str, metas: List[SkillMeta]) -> Dict[str, Any]:
    """首选使用大模型从候选SkillMeta中选择技能；无法确定时由上层回退。

    返回结构：{"mode": "none|single|multi", "selected": [name,...], "reason": str}
    - none：模型判断不需要技能（如闲聊）
    - single/multi：选择一个或多个技能名
    """
    # 仅注入必要字段，避免token浪费
    candidates = [
        {
            "name": m.name,
            "summary": (m.summary or ""),
            "enabled": bool(getattr(m, "enabled", True)),
        }
        for m in (metas or [])
        if getattr(m, "enabled", True)
    ]
    if not candidates or not (user_text or "").strip():
        return {"mode": "none", "selected": [], "reason": "空输入或无候选"}

    system_prompt = (
        "你是技能选择器。给定用户输入与候选技能列表(仅name/summary)，"
        "请基于语义匹配选择最合适的技能；如果是闲聊/问候等无需技能，选择none。"
        "技能可以选择多个，当用户需求需要同时满足多个技能时，mode选择multi。"
        "仅输出JSON，不要额外文本，不要使用Markdown代码块。格式："
        "{\"mode\": \"none|single|multi\", \"selected\": [\"技能名\"...], \"reason\": \"简短理由\"}。"
    )
    content = (
        "[用户输入]\n" + (user_text or "") + "\n\n" +
        "[候选技能]\n" + json.dumps(candidates, ensure_ascii=False)
    )
    messages = [
        {"role": "system", "content": system_prompt},
        {"role": "user", "content": content},
    ]
    try:
        resp = await ai_client.chat_async(messages, temperature=0.0)
        raw = (getattr(resp, "content", "") or "").strip()
        data = _safe_json_loads(raw)
        mode = str(data.get("mode", "")).strip().lower()
        selected = [str(x).strip() for x in (data.get("selected") or []) if str(x).strip()]
        reason = str(data.get("reason", "")).strip()
        if mode not in {"none", "single", "multi"}:
            # 简单纠错：根据数量修正模式
            mode = "none" if not selected else ("single" if len(selected) == 1 else "multi")
        return {"mode": mode, "selected": selected, "reason": reason, "raw": raw}
    except Exception:
        # 失败时由上层回退
        return {"mode": "none", "selected": [], "reason": "模型选择失败"}


def select_skills_by_model_sync(ai_client: Any, user_text: str, metas: List[SkillMeta]) -> Dict[str, Any]:
    """同步版本：使用 AIClient.chat 进行技能选择，便于在同步事件中调用。

    返回结构同异步版本。
    """
    candidates = [
        {
            "name": m.name,
            "summary": (m.summary or ""),
            "enabled": bool(getattr(m, "enabled", True)),
        }
        for m in (metas or [])
        if getattr(m, "enabled", True)
    ]
    if not candidates or not (user_text or "").strip():
        return {"mode": "none", "selected": [], "reason": "空输入或无候选"}

    system_prompt = (
        "你是技能选择器。给定用户输入与候选技能列表(仅name/summary)，"
        "请基于语义匹配选择最合适的技能；如果是闲聊/问候等无需技能，选择none。"
        "技能可以选择多个，当用户需求需要同时满足多个技能时，mode选择multi。"
        "仅输出JSON，不要额外文本，不要使用Markdown代码块。格式："
        "{\"mode\": \"none|single|multi\", \"selected\": [\"技能名\"...], \"reason\": \"简短理由\"}。"
    )
    content = (
        "[用户输入]\n" + (user_text or "") + "\n\n" +
        "[候选技能]\n" + json.dumps(candidates, ensure_ascii=False)
    )
    messages = [
        {"role": "system", "content": system_prompt},
        {"role": "user", "content": content},
    ]
    try:
        logger.info(f"select_skills_by_model_sync: messages={messages}")
        resp = ai_client.chat(messages, temperature=0.0)
        raw = (getattr(resp, "content", "") or "").strip()
        logger.info(f"select_skills_by_model_sync: raw={raw}")
        data = _safe_json_loads(raw)
        mode = str(data.get("mode", "")).strip().lower()
        selected = [str(x).strip() for x in (data.get("selected") or []) if str(x).strip()]
        reason = str(data.get("reason", "")).strip()
        if mode not in {"none", "single", "multi"}:
            mode = "none" if not selected else ("single" if len(selected) == 1 else "multi")
        return {"mode": mode, "selected": selected, "reason": reason, "raw": raw}
    except Exception:
        return {"mode": "none", "selected": [], "reason": "模型选择失败"}


"""
搜索功能的Function Call包装器
"""

import json
import os
from datetime import datetime
import re
from typing import Optional, Union, List, Dict
from ketacli.sdk.ai.function_call import function_registry
from ketacli.sdk.ai.client import AIClient
from ketacli.sdk.base.search import search_spl, search_spl_meta, search_summary
from ketacli.sdk.output.output import search_result_output, list_output, get_asset_output, make_table
from ketacli.sdk.output.format import format_table
from ketacli.sdk.request.list import list_assets_request
from ketacli.sdk.base.client import request_get
from rich.console import Console
import logging

console = Console(markup=False)
logger = logging.getLogger("ketacli.textual.ai_helpers")

def validate_spl_value_quotes(spl: str) -> None:
    """通用校验：SPL 中所有出现“字段值”的场景必须使用双引号。

    原则：
    - 字段名用单引号（例如：'host'）。
    - 字段值用双引号（例如："10-0-1"，包含双引号时可用三双引号）。
    - 校验范围覆盖：
      * search/search2 主段中的条件（如 repo、start、end 及任意 key=value）
      * where 子句中的比较、like/contains/match、in/not in 等
      * 其他命令中的赋值（如 span="10m" 等）
    - 避免误报：允许函数对字段名的单引号（如 avg('field')）、by/fields/as 后的字段名单引号、作为左值的单引号字段名（'field'=...）。
    出现违规时抛出 ValueError 并提供修复建议片段。
    """
    try:
        violations = []

        # 1) 全局：任何 key='value' 的赋值均违规
        for m in re.finditer(r"\b[\w.]+\s*=\s*'[^']*'", spl):
            frag = m.group(0)
            corrected = re.sub(r"=\s*'([^']*)'", r'="\1"', frag)
            violations.append((frag, corrected))

        # 2) where 子句：函数与比较操作的值不得为单引号
        for segment in spl.split("|"):
            seg = segment.strip()
            if not seg:
                continue
            is_where = seg.lower().startswith("where")
            expr = seg[len("where"):].strip() if is_where else seg

            # like/contains/match 第二个参数不应为单引号
            for pattern in [
                r"\blike\s*\(\s*[^,]+,\s*'[^']*'\s*\)",
                r"\bcontains\s*\(\s*[^,]+,\s*'[^']*'\s*\)",
                r"\bmatch\s*\(\s*[^,]+,\s*'[^']*'\s*\)"
            ]:
                for m in re.finditer(pattern, expr, flags=re.IGNORECASE):
                    frag = m.group(0)
                    # 仅替换第二个参数的引号，保留字段名的单引号
                    corrected = re.sub(
                        r"\b(like|contains|match)\s*\(\s*([^,]+),\s*'([^']*)'\s*\)",
                        r'\1(\2, "\3")',
                        frag,
                        flags=re.IGNORECASE,
                    )
                    violations.append((frag, corrected))

            # in/not in 列表值不得为单引号
            for m in re.finditer(r"\b(?:in|not\s+in)\s*\(\s*'[^']*'(?:\s*,\s*'[^']*')*\s*\)", expr, flags=re.IGNORECASE):
                frag = m.group(0)
                corrected = re.sub(r"'([^']*)'", r'"\1"', frag)
                violations.append((frag, corrected))

            # 等值/不等比较右值不得为单引号
            for m in re.finditer(r"(?:=|!=|<>|>=|<=|=~|!~|~)\s*'[^']*'", expr):
                frag = m.group(0)
                corrected = frag.replace("'", '"')
                violations.append((frag, corrected))

        # 3) search/search2 主段：裸单引号文本（自由文本值）不得使用单引号
        first_segment = spl.split("|")[0].strip()
        if first_segment.lower().startswith(("search2", "search")):
            for m in re.finditer(r"'[^']*'", first_segment):
                frag = m.group(0)
                start, end = m.start(), m.end()
                # 允许的上下文：函数参数、by/fields/as 后的字段名、作为左值的字段名（后面紧跟=）
                prefix = first_segment[max(0, start-30):start]
                suffix = first_segment[end:min(len(first_segment), end+10)]

                allowed_prefix_keywords = ["by", "fields", "as"]
                if any(re.search(rf"\b{kw}\s*$", prefix) for kw in allowed_prefix_keywords):
                    continue
                if re.search(r"\([\s]*$", prefix):  # 函数左括号紧邻
                    continue
                if re.match(r"^\s*=", suffix):  # 左值场景：'field'=
                    continue
                # 允许作为操作符左操作数：'field' in / not in / 比较运算符
                if re.match(r"^\s*(in|not\s+in|=|!=|<>|>=|<=|=~|!~|~)\b", suffix, flags=re.IGNORECASE):
                    continue

                # 其他场景视为自由文本值，应改为双引号
                corrected = frag.replace("'", '"')
                violations.append((frag, corrected))

        # 4) 管道规则：where 等算子必须通过管道符引入（缺少时给出修复建议）
        try:
            stage_cmds = ["where"]
            for cmd in stage_cmds:
                if re.search(rf"\b{cmd}\b", first_segment, flags=re.IGNORECASE) and not re.match(rf"^\s*{cmd}\b", first_segment, flags=re.IGNORECASE):
                    for m in re.finditer(rf"\b{cmd}\b", first_segment, flags=re.IGNORECASE):
                        frag = first_segment[max(0, m.start()-30):min(len(first_segment), m.end()+30)]
                        corrected = first_segment[:m.start()].rstrip() + " | " + first_segment[m.start():]
                        violations.append((frag, corrected))
        except Exception:
            pass

        if violations:
            lines = [
                "SPL语法错误：字段值必须使用双引号或 where 等算子前缺少管道符",
                "建议修复片段："
            ]
            for frag, corrected in violations[:10]:
                lines.append(f"- {frag} -> {corrected}")
            if len(violations) > 10:
                lines.append(f"... 共发现 {len(violations)} 处")
            lines.append("规则：字段名用单引号，字段值用双引号（或三双引号）；where 等算子必须通过管道符 '|' 引入。")
            raise ValueError("\n".join(lines))
    except ValueError:
        # 违反规则时应向上抛出，让调用方中断执行并显示错误
        raise
    except Exception as e:
        # 校验实现异常时不影响正常查询，但记录到控制台便于排查
        console.print(f"[yellow]SPL语法校验异常：{str(e)}[/yellow]")



@function_registry.register(
    name="search_data",
    description=f"在KetaDB中搜索指标数据，支持SPL查询语言。请先使用`get_docs `函数获取SPL语法参考文档。",
    parameters={
        "type": "object",
        "properties": {
            "spl": {
                "type": "string",
                "description": "SPL查询语句，必须符合SPL语法规范。"
            },
            "limit": {
                "type": "integer",
                "description": "返回结果数量限制",
                "default": 100
            },
            "format_type": {
                "type": "string",
                "description": "输出格式 (text, json, csv, toon)",
                "default": "toon"
            },
            "output_file": {
                "type": "string",
                "description": "输出文件路径；提供则将结果写入文件"
            },
        },
        "required": ["spl"]
    }
)
def search_data(spl: str, limit: int = 100, format_type: str = "toon", output_file: Optional[str] = None) -> str:
    """搜索KetaDB中的数据"""

    try:
        spl = preflight_spl_fix(spl)
    except Exception:
        pass
    try:
        validate_mstats_syntax(spl)
    except ValueError as ve:
        return str(ve)
    except Exception:
        pass
    
    # 先执行查询；仅在结果为空或查询/解析出错时触发语法校验并返回错误信息
    try:
        resp = search_spl(spl=spl, limit=limit)
    except Exception as se:
        # 查询失败时，尝试语法校验；若存在引号违规则返回更友好的错误提示
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"搜索失败: {str(se)}, 请调用`get_docs log_search_syntax`函数获取SPL语法参考文档以确认查询语句是否正确。"

    # 构造结果输出对象可能抛错（返回结构异常等），同样走校验兜底
    try:
        result_output = search_result_output(resp)
    except Exception as pe:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"结果解析失败: {str(pe)}"

    # 正常格式化输出（有数据时直接返回结果）
    if format_type not in ["json", "csv", "text", "toon"]:
        format_type = "toon"

    text = result_output.get_formatted_string(format_type)

    # 结果为空时才做语法校验，提示可能的引号问题；无违规则返回空结果文本
    try:
        rows_count = len(getattr(result_output, "rows", []) or [])
    except Exception:
        rows_count = 0
    if rows_count == 0:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        # 返回友好提示：建议放宽/减少查询条件
        return (
            "未查询到数据。建议减少或放宽查询条件：尝试扩大时间范围（调整 `start`）、减少 搜索条件过滤项等！"
        )
    # 写文件输出
    if output_file and isinstance(output_file, str) and len(output_file.strip()) > 0:
        try:
            dir_name = os.path.dirname(output_file)
            if dir_name:
                os.makedirs(dir_name, exist_ok=True)
            with open(output_file, "w", encoding="utf-8") as f:
                f.write(text)
            return f"[green]搜索结果已写入: {output_file}[/green]"
        except Exception as fe:
            return f"写入文件失败: {str(fe)}"

    if rows_count == limit:
        text += "\n注意：返回数量等于limit，可能还有更多数据，建议调整limit参数获取更多数据或者通过span等参数调整采样"
    return text


@function_registry.register(
    name="search_metadata",
    description="获取SPL查询的元数据信息",
    parameters={
        "type": "object",
        "properties": {
            "spl": {
                "type": "string",
                "description": "SPL查询语句"
            },
            "output_file": {
                "type": "string",
                "description": "输出文件路径；提供则将结果写入文件"
            }
        },
        "required": ["spl"]
    }
)
def search_metadata(spl: str, output_file: Optional[str] = None) -> str:
    """获取SPL查询的元数据"""
    try:
        if "show" in spl:
            return "show 命令不支持获取元数据，请直接使用search_data_for_metric函数获取指标元数据"
        meta = search_spl_meta(spl)
        table = get_asset_output(resp=meta)
        text = format_table(table, format="toon")
        if output_file and isinstance(output_file, str) and len(output_file.strip()) > 0:
            try:
                dir_name = os.path.dirname(output_file)
                if dir_name:
                    os.makedirs(dir_name, exist_ok=True)
                with open(output_file, "w", encoding="utf-8") as f:
                    f.write(text)
                console.print(f"[green]元数据(TOON)已写入: {output_file}[/green]")
            except Exception as fe:
                console.print(f"[yellow]写入文件失败: {str(fe)}[/yellow]")
        return text
    except Exception as e:
        return f"获取元数据失败: {str(e)}"


@function_registry.register(
    name="get_repo_fields",
    description=f"""获取指定仓库的字段信息和数据摘要。""",
    parameters={
        "type": "object",
        "properties": {
            "repository": {
                "type": "string",
                "description": "目标仓库名称"
            },
            "limit": {
                "type": "integer",
                "description": "返回的字段数量",
                "default": 10
            }
        },
        "required": ["repository"]
    }
)
def get_repo_fields(repository: str, limit: int = 20) -> str:
    """获取指定仓库的字段信息"""
    get_fields_spl = f"search2 repo=\"{repository}\""
    
    try:
        # 构建SPL查询，将时间参数直接嵌入到查询中
        # 调用search_summary获取字段信息，不传递时间参数（因为已经在SPL中了）
        fields = list(search_summary(spl=get_fields_spl).keys())
        # return fields
        # 格式化返回结果
        field_values = {}
        # 初始化计数器和有效字段计数器
        n = 0
        valid_fields_count = 0
        f = open("repo_fields.txt", "a", encoding="utf-8")
        # 遍历所有字段
        while n < len(fields) and valid_fields_count < limit:
            field = fields[n]
            n += 1
            f.write(f"{field}\n")
            
            # 跳过系统字段
            if field in ("_indexTime", "_time", "_raw", "_signature_id", "linenum", "repo"):
                continue
                
            get_fields_value_spl = f"""search2 start="-1h" repo="{repository}"
| where isNotNull('{field}')    
| stats count() as countKey1 by '{field}' with count() as total, distinct('{field}', 0) as distinct
| sort 3 by countKey1
| eval value='{field}', key="{field}"
| fields key, value, countKey1
"""
            rsp = search_spl(spl=get_fields_value_spl)["rows"]
            if not rsp:
                continue
            for item in rsp:
                if item[0] not in field_values:
                    field_values[item[0]] = {}
                field_values[item[0]][item[1]] = str(item[2])[:50] 
            # 增加有效字段计数
            valid_fields_count += 1
        
        rows = []
        rows.append(["repository", repository, None])
        rows.append(["total_fields", len(fields), None])
        rows.append(["return_fields", limit, None])
        for field_name, values in field_values.items():
            for val, cnt in values.items():
                rows.append([field_name, val, cnt])
        table = make_table(["field", "value", "count"], rows)
        text = table.get_formatted_string("toon")
        return text
        
    except Exception as e:
        import traceback
        return f"获取仓库字段信息失败: {str(e)}\n{traceback.format_exc()}"
def preflight_spl_fix(spl: str) -> str:
    try:
        s = spl or ""
        s = s.replace("｜", "|")
        s = re.sub(r"\b([\w.]+)\s*=\s*'([^']*)'", r'\1="\2"', s)
        first_segment = s.split("|")[0]
        if re.search(r"\bwhere\b", first_segment, flags=re.IGNORECASE) and not re.match(r"^\s*where\b", first_segment, flags=re.IGNORECASE):
            s = re.sub(r"(.*?)(\bwhere\b)", lambda m: m.group(1).rstrip()+" | "+m.group(2), first_segment, flags=re.IGNORECASE) + ("|"+"|".join(s.split("|")[1:]) if "|" in s else "")
        s = re.sub(r"\b(?:in|not\s+in)\s*\(([^)]*)\)", lambda m: "in("+re.sub(r"'([^']*)'", r'"\\1"', m.group(1))+")" if m.group(0).lower().startswith("in") else "not in("+re.sub(r"'([^']*)'", r'"\\1"', m.group(1))+")", s, flags=re.IGNORECASE)
        return s
    except Exception:
        return spl

def validate_mstats_syntax(spl: str) -> None:
    t = (spl or "").strip()
    tl = t.lower()
    if not tl.startswith("mstats"):
        if "mstats" in tl:
            pass
        else:
            return
    if re.search(r"\bsearch2\b", tl):
        raise ValueError("SPL语法错误：指标查询不得与search2混用，请使用mstats并在开头给出时间参数")
    if not re.match(r"^\s*mstats\s+", tl):
        raise ValueError("SPL语法错误：mstats必须作为查询开头")
    if not re.search(r"mstats\s+start=|mstats\s+end=", tl):
        raise ValueError("SPL语法错误：mstats后需紧跟时间参数start/end")
    for m in re.finditer(r"\b(avg|sum|min|max|rate|topseries|hperc|distinct)\b", tl):
        pass
    if re.search(r"\bsort\s+by\b", tl) is None and re.search(r"\btopseries\b", tl):
        raise ValueError("SPL语法错误：使用topseries时建议配合sort by 指定排序字段")


@function_registry.register(
    name="simple_search",
    description="根据参数组装日志SPL并搜索数据。支持仓库、过滤条件、排序与限制。",
    parameters={
        "type": "object",
        "properties": {
            "repository": {"type": "string", "description": "仓库名"},
            "start": {"type": "string", "description": "开始时间，如 -1h、-d@d"},
            "end": {"type": "string", "description": "结束时间，如 @h、@d"},
            "filters": {
                "type": "array",
                "description": "过滤条件列表，支持与或非",
                "items": {
                    "type": "object",
                    "properties": {
                        "field": {"type": "string", "description": "字段名"},
                        "op": {"type": "string", "description": "操作符，如 =, !=, >=, <=, >, <, in, not in"},
                        "value": {"description": "值(非in时)", "anyOf": [{"type": "string"}, {"type": "number"}]},
                        "values": {"type": "array", "items": {"anyOf": [{"type": "string"}, {"type": "number"}]}},
                        "text": {"type": "string", "description": "全文匹配文本"},
                        "not": {"type": "boolean", "description": "取反"}
                    }
                }
            },
            "stats": {
                "type": "array",
                "description": "聚合算子列表，如 count()/sum('f') 等",
                "items": {
                    "type": "object",
                    "properties": {
                        "func": {"type": "string", "description": "聚合函数，如 count,sum,avg,min,max,distinct,percentile"},
                        "field": {"type": "string", "description": "聚合字段（count可省略）"},
                        "alias": {"type": "string", "description": "别名"},
                        "args": {"type": "array", "items": {"anyOf": [{"type": "string"}, {"type": "number"}]}, "description": "额外参数，如percentile的百分位"}
                    }
                }
            },
            "group_by": {
                "type": "array",
                "items": {"type": "string"},
                "description": "分组字段列表"
            },
            "logic": {"type": "string", "enum": ["AND", "OR"], "default": "AND", "description": "过滤条件组合逻辑"},
            "sort_field": {"type": "string", "description": "排序字段"},
            "sort_order": {"type": "string", "enum": ["asc", "desc"], "default": "desc", "description": "排序顺序"},
            "limit": {"type": "integer", "default": 100, "description": "数量限制"},
            "format_type": {"type": "string", "default": "toon", "description": "输出格式(text,json,csv,toon)，优先选择toon"},
            "output_file": {"type": "string", "description": "写入文件路径"}
        },
        "required": ["repository"]
    }
)
def simple_search(
    repository: str,
    start: Optional[str] = None,
    end: Optional[str] = None,
    filters: Optional[list] = None,
    stats: Optional[list] = None,
    group_by: Optional[list] = None,
    logic: str = "AND",
    sort_field: Optional[str] = None,
    sort_order: str = "desc",
    limit: int = 100,
    format_type: str = "toon",
    output_file: Optional[str] = None,
) -> str:
    def is_number(v: Union[str, int, float]) -> bool:
        if isinstance(v, (int, float)):
            return True
        if isinstance(v, str):
            return re.fullmatch(r"-?\d+(?:\.\d+)?", v) is not None
        return False

    def quote_val(v: Union[str, int, float]) -> str:
        if is_number(v):
            return str(v)
        s = str(v)
        if '"' in s:
            return f'"""{s}"""'
        return f'"{s}"'

    def build_condition(fobj: dict) -> str:
        if not isinstance(fobj, dict):
            return ""
        negate = bool(fobj.get("not"))
        if fobj.get("text") and not fobj.get("field"):
            expr = quote_val(fobj.get("text"))
            return f"NOT ({expr})" if negate else expr
        field = fobj.get("field")
        op = (fobj.get("op") or "=").strip().lower()
        if not field:
            return ""
        l = f"'{field}'"
        if op in ("in", "not in"):
            vals = fobj.get("values") or []
            parts = []
            for vv in vals:
                parts.append(quote_val(vv))
            expr = f"{l} {op} (" + ",".join(parts) + ")"
            return f"NOT ({expr})" if negate else expr
        if op in ("contains", "like"):
            tv = fobj.get("value")
            if tv is None:
                return ""
            patt = str(tv)
            if op == "contains" and not (patt.startswith("*") or patt.endswith("*")):
                patt = f"*{patt}*"
            expr = quote_val(patt)
            return f"NOT ({expr})" if negate else expr
        rv = fobj.get("value")
        if rv is None:
            return ""
        expr = f"{l}{op if op in ['=','!=','>=','<=','>','<','=~','!~','~'] else '='}{quote_val(rv)}"
        return f"NOT ({expr})" if negate else expr

    segments = ["search2"]
    time_parts = []
    if isinstance(start, str) and start.strip():
        time_parts.append(f'start="{start.strip()}"')
    if isinstance(end, str) and end.strip():
        time_parts.append(f'end="{end.strip()}"')
    if time_parts:
        segments.append(" ".join(time_parts))
    segments.append(f'repo="{repository}"')

    conds = []
    if isinstance(filters, list) and filters:
        for it in filters:
            c = build_condition(it)
            if c:
                conds.append(c)
    if conds:
        joiner = f" {('OR' if (logic or 'AND').upper() == 'OR' else 'AND')} "
        segments.append("(" + joiner.join(conds) + ")")

    spl = " ".join(segments)
    pipes = []
    agg_parts = []
    if isinstance(stats, list) and stats:
        for agg in stats:
            if not isinstance(agg, dict):
                continue
            func = str(agg.get("func") or "").strip()
            if not func:
                continue
            fl = str(agg.get("field") or "").strip()
            args = agg.get("args") or []
            func_l = func.lower()
            if func_l == "count":
                expr = "count()"
            else:
                if not fl and func_l not in ("earliest", "latest"):
                    continue
                if args and isinstance(args, list) and len(args) > 0:
                    arg_strs = []
                    for a in args:
                        if isinstance(a, (int, float)) or (isinstance(a, str) and re.fullmatch(r"-?\d+(?:\.\d+)?", a)):
                            arg_strs.append(str(a))
                        else:
                            s = str(a)
                            arg_strs.append(f'"{s}"' if '"' not in s else f'"""{s}"""')
                    expr = f"{func}('{fl}', " + ",".join(arg_strs) + ")"
                else:
                    if fl:
                        expr = f"{func}('{fl}')"
                    else:
                        expr = f"{func}()"
            alias = str(agg.get("alias") or "").strip()
            if alias:
                expr = expr + f" as '{alias}'"
            agg_parts.append(expr)
    if agg_parts:
        by_part = ""
        if isinstance(group_by, list) and group_by:
            gb = [f"'{str(g).strip()}'" for g in group_by if str(g).strip()]
            if gb:
                by_part = " by " + ", ".join(gb)
        pipes.append("stats " + ", ".join(agg_parts) + by_part)
    if sort_field and isinstance(sort_field, str) and sort_field.strip():
        order = (sort_order or "desc").lower()
        if order not in ("asc", "desc"):
            order = "desc"
        pipes.append(f"sort by '{sort_field.strip()}' {order}")
    pipes.append(f"limit {int(limit) if isinstance(limit, int) else 100}")
    if pipes:
        spl = spl + " | " + " | ".join(pipes)

    try:
        spl = preflight_spl_fix(spl)
    except Exception:
        pass
    try:
        validate_spl_value_quotes(spl)
    except ValueError as ve:
        return str(ve)
    except Exception:
        pass

    logger.info(f"生成的执行SPL: {spl}")

    try:
        resp = search_spl(spl=spl, limit=limit)
    except Exception as se:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"搜索失败: {str(se)}, 请调用`get_docs log_search_syntax`函数获取SPL语法参考文档以确认查询语句是否正确。"

    try:
        result_output = search_result_output(resp)
    except Exception as pe:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"结果解析失败: {str(pe)}"

    if format_type not in ["json", "csv", "text", "toon"]:
        format_type = "toon"
    text = result_output.get_formatted_string(format_type)
    try:
        rows_count = len(getattr(result_output, "rows", []) or [])
    except Exception:
        rows_count = 0
    if rows_count == 0:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return (
            "未查询到数据。建议减少或放宽查询条件：尝试扩大时间范围（调整 `start`）、减少 搜索条件过滤项等！"
        )
    if output_file and isinstance(output_file, str) and len(output_file.strip()) > 0:
        try:
            dir_name = os.path.dirname(output_file)
            if dir_name:
                os.makedirs(dir_name, exist_ok=True)
            with open(output_file, "w", encoding="utf-8") as f:
                f.write(text)
            return f"搜索结果已写入: {output_file}"
        except Exception as fe:
            return f"写入文件失败: {str(fe)}"
    if rows_count == limit:
        text += "\n注意：返回数量等于limit，可能还有更多数据，建议调整limit参数获取更多数据或者通过span等参数调整采样"
    return text


@function_registry.register(
    name="search_traces_by_traceid",
    description="根据traceid查询TRACES链路数据，支持指定返回字段。",
    parameters={
        "type": "object",
        "properties": {
            "traceid": {"type": "string", "description": "链路ID"},
            "start": {"type": "string", "description": "开始时间", "default": "-3h"},
            "return_fields": {
                "type": "array",
                "description": "需要返回的字段列表",
                "items": {"type": "string"}
            },
            "limit": {"type": "integer", "description": "返回数量限制", "default": 100},
            "format_type": {"type": "string", "description": "输出格式(text,json,csv,toon)", "default": "toon"},
            "output_file": {"type": "string", "description": "输出文件路径"}
        },
        "required": ["traceid"]
    }
)
def search_traces_by_traceid(
    traceid: str,
    start: str = "-3h",
    return_fields: Optional[list] = None,
    limit: int = 100,
    format_type: str = "toon",
    output_file: Optional[str] = None,
) -> str:
    segments = [
        "search2",
        f'start="{start.strip()}"' if isinstance(start, str) and start.strip() else 'start="-3h"',
        '_repo_type="TRACES"',
        f'(\'traceId\'="*{traceid}*")'
    ]
    spl = " ".join(segments)
    pipes = [
        "eval duration = durationNanos / 1000000",
        'eval attrs = arr_join(attrs, "[:]"), resource.attrs = arr_join(resource.attrs, "[:]")',
    ]
    if isinstance(return_fields, list) and len(return_fields) > 0:
        fs = ", ".join([str(f).strip() for f in return_fields if str(f).strip()])
        if fs:
            pipes.append(f"fields {fs}")
    pipes.append(f"limit {int(limit) if isinstance(limit, int) else 100}")
    spl = spl + " | " + " | ".join(pipes)

    try:
        spl = preflight_spl_fix(spl)
    except Exception:
        pass
    try:
        validate_spl_value_quotes(spl)
    except ValueError as ve:
        return str(ve)
    except Exception:
        pass

    try:
        resp = search_spl(spl=spl, limit=limit)
    except Exception as se:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"搜索失败: {str(se)}"

    try:
        result_output = search_result_output(resp)
    except Exception as pe:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"结果解析失败: {str(pe)}"

    if format_type not in ["json", "csv", "text", "toon"]:
        format_type = "toon"
    text = result_output.get_formatted_string(format_type)
    try:
        rows_count = len(getattr(result_output, "rows", []) or [])
    except Exception:
        rows_count = 0
    if rows_count == 0:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return "未查询到数据。请确认traceid正确，并尝试扩大时间范围。"
    if output_file and isinstance(output_file, str) and len(output_file.strip()) > 0:
        try:
            dir_name = os.path.dirname(output_file)
            if dir_name:
                os.makedirs(dir_name, exist_ok=True)
            with open(output_file, "w", encoding="utf-8") as f:
                f.write(text)
            return f"搜索结果已写入: {output_file}"
        except Exception as fe:
            return f"写入文件失败: {str(fe)}"
    if rows_count == limit:
        text += "\n注意：返回数量等于limit，可能还有更多数据，可调整limit或时间范围。"
    return text


@function_registry.register(
    name="search_trace_exceptions_by_traceid",
    description="根据traceid查询TRACES中的异常事件详情，支持指定返回字段。",
    parameters={
        "type": "object",
        "properties": {
            "traceid": {"type": "string", "description": "链路ID"},
            "start": {"type": "string", "description": "开始时间", "default": "-3h"},
            "return_fields": {
                "type": "array",
                "description": "需要返回的字段列表",
                "items": {"type": "string"}
            },
            "limit": {"type": "integer", "description": "返回数量限制", "default": 100},
            "format_type": {"type": "string", "description": "输出格式(text,json,csv,toon)", "default": "toon"},
            "output_file": {"type": "string", "description": "输出文件路径"}
        },
        "required": ["traceid"]
    }
)
def search_trace_exceptions_by_traceid(
    traceid: str,
    start: str = "-3h",
    return_fields: Optional[list] = None,
    limit: int = 100,
    format_type: str = "toon",
    output_file: Optional[str] = None,
) -> str:
    segments = [
        "search2",
        f'start="{start.strip()}"' if isinstance(start, str) and start.strip() else 'start="-3h"',
        '_repo_type="TRACES"',
        f'(\'traceId\'="*{traceid}*")'
    ]
    spl = " ".join(segments)
    pipes = [
        "mvexpand events",
        'eval events.name = jsonpath(events, "name")',
        'where events.name = "exception"',
        'eval exception.type = jsonpath(events, """attributes[\"exception.type\"]"""), exception.message = jsonpath(events, """attributes[\"exception.message\"]"""), exception.stacktrace = jsonpath(events, """attributes[\"exception.stacktrace\"]"""), exception.escaped = jsonpath(events, """attributes[\"exception.escaped\"]""")',
        'eval timeUnixNano = jsonpath(events, "timeUnixNano"), _time = timeUnixNano / 1000000',
        'eval _raw = if (exception.message != "", concat(exception.type, ": ", exception.message), exception.type)',
        'rename kind as span.kind, name as span.name',
        "fields _time, _raw, timeUnixNano, env, service, servicetype, version, instance, host, scope.name, scope.version, span.kind, span.name, traceId, spanId, parentSpanId, exception.type, exception.message, exception.stacktrace, exception.escaped"
    ]
    if isinstance(return_fields, list) and len(return_fields) > 0:
        fs = ", ".join([str(f).strip() for f in return_fields if str(f).strip()])
        if fs:
            pipes.append(f"fields {fs}")
    pipes.append(f"limit {int(limit) if isinstance(limit, int) else 100}")
    spl = spl + " | " + " | ".join(pipes)

    try:
        spl = preflight_spl_fix(spl)
    except Exception:
        pass
    try:
        validate_spl_value_quotes(spl)
    except ValueError as ve:
        return str(ve)
    except Exception:
        pass

    try:
        resp = search_spl(spl=spl, limit=limit)
    except Exception as se:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"搜索失败: {str(se)}"

    try:
        result_output = search_result_output(resp)
    except Exception as pe:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return f"结果解析失败: {str(pe)}"

    if format_type not in ["json", "csv", "text", "toon"]:
        format_type = "toon"
    text = result_output.get_formatted_string(format_type)
    try:
        rows_count = len(getattr(result_output, "rows", []) or [])
    except Exception:
        rows_count = 0
    if rows_count == 0:
        try:
            validate_spl_value_quotes(spl)
        except ValueError as ve:
            return str(ve)
        return "未查询到异常事件。请确认traceid正确，并尝试扩大时间范围。"
    if output_file and isinstance(output_file, str) and len(output_file.strip()) > 0:
        try:
            dir_name = os.path.dirname(output_file)
            if dir_name:
                os.makedirs(dir_name, exist_ok=True)
            with open(output_file, "w", encoding="utf-8") as f:
                f.write(text)
            return f"搜索结果已写入: {output_file}"
        except Exception as fe:
            return f"写入文件失败: {str(fe)}"
    if rows_count == limit:
        text += "\n注意：返回数量等于limit，可能还有更多异常事件，可调整limit或时间范围。"
    return text


@function_registry.register(
    name="simple_metric_search",
    description="根据指标名称与聚合方式生成 mstats 查询并返回结果。",
    parameters={
        "type": "object",
        "properties": {
            "metrics": {
                "type": "array",
                "description": "指标聚合列表",
                "items": {
                    "type": "object",
                    "properties": {
                        "name": {"type": "string", "description": "指标名称"},
                        "func": {"type": "string", "description": "聚合函数，如 avg,sum,min,max,distinct,percentile,hperc,sumRate"},
                        "alias": {"type": "string", "description": "别名"},
                        "args": {"type": "array", "items": {"anyOf": [{"type": "number"}, {"type": "string"}]}, "description": "函数额外参数"}
                    },
                    "required": ["name", "func"]
                }
            },
            "start": {"type": "string", "description": "开始时间，如 -1h、-d@d", "default": "-1h"},
            "end": {"type": "string", "description": "结束时间，如 @h、@d"},
            "span": {"type": "string", "description": "时间粒度，如 1m、5m"},
            "timeshift": {"type": "string", "description": "时间偏移，如 -1h"},
            "filters": {
                "type": "array",
                "description": "标签过滤条件（用于 where 段）",
                "items": {
                    "type": "object",
                    "properties": {
                        "field": {"type": "string"},
                        "op": {"type": "string", "description": "=, !=, like, in, not in"},
                        "value": {"anyOf": [{"type": "string"}, {"type": "number"}]},
                        "values": {"type": "array", "items": {"anyOf": [{"type": "string"}, {"type": "number"}]}},
                        "not": {"type": "boolean"}
                    }
                }
            },
            "logic": {"type": "string", "enum": ["AND", "OR"], "default": "AND"},
            "group_by": {"type": "array", "items": {"type": "string"}},
            "sort_field": {"type": "string"},
            "sort_order": {"type": "string", "enum": ["asc", "desc"], "default": "desc"},
            "limit": {"type": "integer", "default": 100},
            "format_type": {"type": "string", "default": "toon", "description": "输出格式(text,json,csv,toon)"},
            "output_file": {"type": "string"}
        },
        "required": ["metrics"]
    }
)
def simple_metric_search(
    metrics: list,
    start: Optional[str] = "-1h",
    end: Optional[str] = None,
    span: Optional[str] = None,
    timeshift: Optional[str] = None,
    filters: Optional[list] = None,
    logic: str = "AND",
    group_by: Optional[list] = None,
    sort_field: Optional[str] = None,
    sort_order: str = "desc",
    limit: int = 100,
    format_type: str = "toon",
    output_file: Optional[str] = None,
) -> str:
    spl = build_mstats_spl(
        metrics=metrics,
        start=start,
        end=end,
        span=span,
        timeshift=timeshift,
        filters=filters,
        logic=logic,
        group_by=group_by,
        sort_field=sort_field,
        sort_order=sort_order,
        limit=limit,
    )

    try:
        spl = preflight_spl_fix(spl)
    except Exception:
        pass
    try:
        validate_mstats_syntax(spl)
    except ValueError as ve:
        return str(ve)
    except Exception:
        pass

    logger.info(f"生成的指标SPL: {spl}")

    try:
        resp = search_spl(spl=spl, limit=limit)
    except Exception as se:
        return f"搜索失败: {str(se)}"
    try:
        result_output = search_result_output(resp)
    except Exception as pe:
        return f"结果解析失败: {str(pe)}"
    if format_type not in ["json", "csv", "text", "toon"]:
        format_type = "toon"
    text = result_output.get_formatted_string(format_type)
    try:
        rows_count = len(getattr(result_output, "rows", []) or [])
    except Exception:
        rows_count = 0
    if rows_count == 0:
        return (
            "未查询到数据。建议调整时间范围（start/end/span）、检查分组字段是否为标签，或放宽过滤条件。SPL: " + spl
        )
    if output_file and isinstance(output_file, str) and len(output_file.strip()) > 0:
        try:
            dir_name = os.path.dirname(output_file)
            if dir_name:
                os.makedirs(dir_name, exist_ok=True)
            with open(output_file, "w", encoding="utf-8") as f:
                f.write(text)
            return f"[green]搜索结果已写入: {output_file}[/green]"
        except Exception as fe:
            return f"[yellow]写入文件失败: {str(fe)}[/yellow]"
    if rows_count == limit:
        text += "\n注意：返回数量等于limit，可能还有更多数据，建议调整limit或span参数。"
    return text

def build_mstats_spl(
    metrics: List[Dict],
    start: Optional[str] = None,
    end: Optional[str] = None,
    span: Optional[str] = None,
    timeshift: Optional[str] = None,
    filters: Optional[List[Dict]] = None,
    logic: str = "AND",
    group_by: Optional[List[str]] = None,
    sort_field: Optional[str] = None,
    sort_order: str = "desc",
    limit: int = 100,
) -> str:
    def is_number(v: Union[str, int, float]) -> bool:
        if isinstance(v, (int, float)):
            return True
        if isinstance(v, str):
            return re.fullmatch(r"-?\d+(?:\.\d+)?", v) is not None
        return False
    def quote_val(v: Union[str, int, float]) -> str:
        if is_number(v):
            return str(v)
        s = str(v)
        return f'"""{s}"""' if '"' in s else f'"{s}"'
    def build_where_condition(c: dict) -> str:
        if not isinstance(c, dict):
            return ""
        field = str(c.get("field") or "").strip()
        if not field:
            return ""
        op = (c.get("op") or "=").strip().lower()
        negate = bool(c.get("not"))
        l = f"'{field}'"
        if op in ("in", "not in"):
            vals = c.get("values") or []
            parts = [f"{l}={quote_val(v)}" for v in vals]
            expr = "(" + " OR ".join(parts) + ")"
            return f"NOT {expr}" if negate or op == "not in" else expr
        if op == "like":
            v = c.get("value")
            if v is None:
                return ""
            expr = f"like({l}, {quote_val(v)})"
            return f"NOT ({expr})" if negate else expr
        v = c.get("value")
        if v is None:
            return ""
        expr = f"{l}{op if op in ['=','!='] else '='}{quote_val(v)}"
        return f"NOT ({expr})" if negate else expr
    segs = ["mstats"]
    time_params = []
    if isinstance(start, str) and start.strip():
        time_params.append(f'start="{start.strip()}"')
    if isinstance(end, str) and end.strip():
        time_params.append(f'end="{end.strip()}"')
    if isinstance(span, str) and span.strip():
        time_params.append(f'span="{span.strip()}"')
    if isinstance(timeshift, str) and timeshift.strip():
        time_params.append(f'timeshift="{timeshift.strip()}"')
    if time_params:
        segs.append(" ".join(time_params))
    agg_exprs = []
    for m in metrics or []:
        if not isinstance(m, dict):
            continue
        name = str(m.get("name") or "").strip()
        func = str(m.get("func") or "").strip()
        if not name or not func:
            continue
        args = m.get("args") or []
        arg_str = ""
        if args:
            arg_parts = []
            for a in args:
                if is_number(a):
                    arg_parts.append(str(a))
                else:
                    s = str(a)
                    arg_parts.append(f'"{s}"' if '"' not in s else f'"""{s}"""')
            arg_str = ", " + ",".join(arg_parts)
        expr = f"{func}('{name}'{arg_str})"
        alias = str(m.get("alias") or "").strip()
        if alias:
            expr = expr + f" as '{alias}'"
        agg_exprs.append(expr)
    by_part = ""
    if isinstance(group_by, list) and group_by:
        gb = [f"'{str(g).strip()}'" for g in group_by if str(g).strip()]
        if gb:
            by_part = " by " + ", ".join(gb)
    if agg_exprs:
        segs.append(" ".join([", ".join(agg_exprs) + by_part]))
    spl = " ".join(segs)
    pipes = []
    where_parts = []
    if isinstance(filters, list) and filters:
        for c in filters:
            wc = build_where_condition(c)
            if wc:
                where_parts.append(wc)
    if where_parts:
        joiner = f" {('OR' if (logic or 'AND').upper() == 'OR' else 'AND')} "
        pipes.append("where " + "(" + joiner.join(where_parts) + ")")
    if sort_field and isinstance(sort_field, str) and sort_field.strip():
        order = (sort_order or "desc").lower()
        if order not in ("asc", "desc"):
            order = "desc"
        pipes.append(f"sort by '{sort_field.strip()}' {order}")
    pipes.append(f"limit {int(limit) if isinstance(limit, int) else 100}")
    if pipes:
        spl = spl + " | " + " | ".join(pipes)
    return spl



"""数据模型和会话管理

包含聊天会话的数据结构和会话管理功能。
"""

import json
import os
import uuid
from typing import Dict, List, Any, Optional
from datetime import datetime
from dataclasses import dataclass, asdict, field


@dataclass
class ChatSession:
    """聊天会话数据结构"""
    session_id: str
    title: str
    created_at: str
    updated_at: str
    messages: List[Dict[str, Any]]
    # 阶段5新增：步骤与工具记录（便于复盘与恢复）
    steps: List[Dict[str, Any]] = field(default_factory=list)
    tools: List[Dict[str, Any]] = field(default_factory=list)
    current_plan_steps: List[str] = field(default_factory=list)
    current_plan_index: int = 0
    current_plan_task_text: str = ""
    
    @classmethod
    def create_new(cls, title: str = None) -> 'ChatSession':
        """创建新会话"""
        now = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        session_id = str(uuid.uuid4())
        if not title:
            title = f"会话 {datetime.now().strftime('%m-%d %H:%M')}"
        
        return cls(
            session_id=session_id,
            title=title,
            created_at=now,
            updated_at=now,
            messages=[]
        )
    
    def update_messages(self, messages: List[Dict[str, Any]]):
        """更新消息列表"""
        self.messages = messages
        self.updated_at = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    
    def get_display_title(self) -> str:
        """获取显示标题"""
        if len(self.messages) > 0:
            # 尝试从第一条用户消息生成标题
            for msg in self.messages:
                if msg.get("role") == "user":
                    content = msg.get("content", "")
                    if content:
                        # 取前20个字符作为标题
                        title = content[:20]
                        if len(content) > 20:
                            title += "..."
                        return title
        return self.title


class SessionManager:
    """会话管理器"""
    
    def __init__(self, storage_dir: str = None):
        if storage_dir is None:
            # 使用用户主目录下的.ketacli文件夹
            home_dir = os.path.expanduser("~")
            storage_dir = os.path.join(home_dir, ".ketacli", "chat_sessions")
        
        self.storage_dir = storage_dir
        os.makedirs(storage_dir, exist_ok=True)
    
    def save_session(self, session: ChatSession) -> bool:
        """保存会话"""
        try:
            file_path = os.path.join(self.storage_dir, f"{session.session_id}.json")
            with open(file_path, 'w', encoding='utf-8') as f:
                json.dump(asdict(session), f, ensure_ascii=False, indent=2)
            return True
        except Exception as e:
            print(f"保存会话失败: {e}")
            return False
    
    def load_session(self, session_id: str) -> Optional[ChatSession]:
        """加载会话"""
        try:
            file_path = os.path.join(self.storage_dir, f"{session_id}.json")
            if not os.path.exists(file_path):
                return None
            
            with open(file_path, 'r', encoding='utf-8') as f:
                data = json.load(f)
            
            return ChatSession(**data)
        except Exception as e:
            print(f"加载会话失败: {e}")
            return None
    
    def list_sessions(self) -> List[ChatSession]:
        """列出所有会话"""
        sessions = []
        try:
            for filename in os.listdir(self.storage_dir):
                if filename.endswith('.json'):
                    session_id = filename[:-5]  # 移除.json后缀
                    session = self.load_session(session_id)
                    if session:
                        sessions.append(session)
            
            # 按更新时间倒序排列
            sessions.sort(key=lambda x: x.updated_at, reverse=True)
        except Exception as e:
            print(f"列出会话失败: {e}")
        
        return sessions
    
    def delete_session(self, session_id: str) -> bool:
        """删除会话"""
        try:
            file_path = os.path.join(self.storage_dir, f"{session_id}.json")
            if os.path.exists(file_path):
                os.remove(file_path)
                return True
            return False
        except Exception as e:
            print(f"删除会话失败: {e}")
            return False



import argcomplete
from mando import main

# 导入所有插件模块
from ketacli.plugins import auth
from ketacli.plugins import asset
from ketacli.plugins import search
from ketacli.plugins import visualization
from ketacli.plugins import data
from ketacli.plugins import config
from ketacli.plugins import interactive
from ketacli.plugins import metric

# 导入mock模块中的函数
from ketacli.plugins.mock import mock_data, mock_log, mock_metrics, generate_and_upload

# 导入测试模块
from ketacli.sdk.test.cli import test_command

# 导入AI模块
from ketacli.plugins.ai_search import *


def start():
    # 确保main被正确初始化
    import argcomplete
    argcomplete.autocomplete(main.parser)
    try:
        argcomplete.autocomplete(main.parser)
        main()
    except Exception:
        from rich.console import Console
        console = Console()
        console.print_exception()


if __name__ == "__main__":
    start()



"""
消息操作组件：复制 + 展开/收起

统一为模型消息与工具消息提供复制与展开/收起操作。
"""

from textual.app import ComposeResult
from textual.containers import Horizontal
from textual.widgets import Button
from textual.reactive import reactive


class MessageActions(Horizontal):
    """消息操作条组件

    - 复制消息内容
    - 展开/收起切换（单个按钮，依据状态变更文案）
    """

    is_expanded = reactive(False)

    def __init__(
        self,
        on_toggle,
        on_copy,
        is_long: bool,
        is_expanded: bool = False,
        unique_id: str = "",
        show_copy: bool = True,
        **kwargs,
    ):
        # 继承 Horizontal，设置容器样式类
        super().__init__(classes="message-actions", **kwargs)
        self.on_toggle = on_toggle
        self.on_copy = on_copy
        self.is_long = is_long
        self.is_expanded = is_expanded
        self.unique_id = unique_id
        self.show_copy = show_copy

    def compose(self) -> ComposeResult:
        """通过 compose 产出子按钮，避免在未挂载时执行 mount 导致错误"""
        if self.show_copy:
            yield Button(
                "📋 复制",
                id=f"copy-actions-{self.unique_id}",
                classes="copy-button",
            )
        if self.is_long:
            label = "收起" if self.is_expanded else "展开"
            yield Button(
                label,
                id=f"toggle-actions-{self.unique_id}",
                classes="expand-button",
            )

    def on_button_pressed(self, event: Button.Pressed) -> None:
        """处理操作条按钮点击"""
        # 复制
        if event.button.id == f"copy-actions-{self.unique_id}":
            if callable(self.on_copy):
                self.on_copy()
            return

        # 展开/收起切换
        if event.button.id == f"toggle-actions-{self.unique_id}":
            new_state = not self.is_expanded
            self.is_expanded = new_state
            if callable(self.on_toggle):
                self.on_toggle(new_state)

            # 直接更新按钮文案，避免重复ID导致的挂载错误
            try:
                toggle_btn = event.button
                toggle_btn.label = "收起" if new_state else "展开"
                toggle_btn.refresh()
            except Exception:
                # 兜底：如果无法直接更新，则查询并更新
                try:
                    btn = self.query_one(
                        f"#toggle-actions-{self.unique_id}", Button
                    )
                    btn.label = "收起" if new_state else "展开"
                    btn.refresh()
                except Exception:
                    pass


import csv
import json
import subprocess
import sys
from pathlib import Path


def run_convert(expr: str, repo: str = "zabbix") -> dict:
    cmd = [
        sys.executable,
        "-m",
        "ketacli",
        "convert",
        "--src",
        "zabbix",
        "--dst",
        "spl_json",
        "--host-field",
        "",
        "--origin-field",
        "",
        "--repo",
        repo,
        "-t",
        expr,
    ]
    proc = subprocess.run(cmd, capture_output=True, text=True)
    if proc.returncode != 0:
        raise RuntimeError(f"convert failed: {proc.stderr}")
    return json.loads(proc.stdout.strip())


def summarize(idx: int, title: str, expr: str, result: dict) -> dict:
    spl = result.get("SPL", "")
    crontab = result.get("crontab", [])
    return {
        # "index": idx,
        "title": title,
        "zabbix": expr,
        "result": {
            "spl": spl,
            "crontab": crontab
        }
        
        # "has_join": ("join type=" in spl),
        # "has_join_tmp": ("eval join_tmp=1" in spl),
        # "need_alert_is_cond0": ("eval need_alert=cond_0" in spl),
        # "crontab_len": len(crontab),
    }


def main():
    csv_path = Path(__file__).parent / "zabbix_roles.csv"
    rows = []
    with csv_path.open("r", newline="", encoding="utf-8") as f:
        reader = csv.reader(f)
        for row in reader:
            if not row:
                continue
            # Expect 2 columns: title, expression
            if len(row) < 2:
                # skip malformed line
                continue
            title = row[0]
            expr = row[1]
            rows.append((title, expr))

    results = []
    for i, (title, expr) in enumerate(rows, start=1):
        try:
            res = run_convert(expr)
            summary = summarize(i, title, expr, res)
        except Exception as e:
            summary = {
                "index": i,
                "title": title,
                "error": str(e),
            }
        results.append(summary)

    print(json.dumps({"results": results}, ensure_ascii=False, indent=2))


if __name__ == "__main__":
    main()


"""全新计划执行器（PlanExecutor）

目标：提供独立的新执行路径，不与旧逻辑混用。
职责：统一步骤执行、工具调用、早停与状态更新。
"""

from typing import List, Dict, Any, Optional
import logging
from datetime import datetime

from ..widgets import ChatHistoryWidget, ModelSelectorWidget, TaskManagerWidget
from .ai_helpers import get_enabled_tools_openai_format, process_tool_calls, needs_tool_call
from ketacli.sdk.textual_chart.utils.chat_stream import safe_notify

logger = logging.getLogger("ketacli.textual")


# 新版标签常量（不依赖旧文件）
NEW_STEP_NEXT_MARKERS = {"[STEP_NEXT]", "[step_next]"}
NEW_STEP_WAIT_MARKERS = {"[STEP_WAIT]", "[step_wait]"}
NEW_STEP_BLOCKED_MARKERS = {"[STEP_BLOCKED]", "[step_blocked]"}
NEW_TASK_DONE_MARKERS = {"[TASK_DONE]", "[task_done]"}
NEW_STEP_RETRY_MARKERS = {"[STEP_RETRY]", "[step_retry]"}


def _has_marker(text: str, markers: set) -> bool:
    try:
        lower = (text or "").lower()
    except Exception:
        lower = ""
    if not lower:
        return False
    return any((m or "").lower() in lower for m in (markers or set()))


def _normalize_tool_calls(tool_calls: Optional[List[Dict[str, Any]]]) -> List[Dict[str, Any]]:
    """简单去重工具调用：以函数名+参数JSON为签名去重。"""
    logger.info(f"[tools] 原始工具调用列表: {tool_calls}")
    res: List[Dict[str, Any]] = []
    seen = set()
    for tc in tool_calls or []:
        fn = (tc or {}).get("function", {})
        name = fn.get("name") or ""
        args = fn.get("arguments") or ""
        sig = f"{name}|{args}"
        if sig in seen:
            continue
        seen.add(sig)
        res.append(tc)
    return res


class PlanExecutor:
    """新的步骤执行器：独立文件中实现，不依赖旧 execute_task_steps。"""

    def __init__(self):
        pass

    async def run(self, app: Any, steps: List[str], original_user_text: str, start_index: int = 0) -> None:
        """执行给定的任务步骤列表。

        - 独立管理状态：`app.plan_status`
        - 统一工具调用：`process_tool_calls`
        - 早停规则：WAIT/BLOCK/DONE 标签
        """
        try:
            setattr(app, "plan_status", "running")
        except Exception:
            pass


        # 组件获取（容错）
        try:
            chat_history = app.query_one("#chat-history", ChatHistoryWidget)
        except Exception:
            chat_history = None
        try:
            model_selector = app.query_one("#model-selector", ModelSelectorWidget)
        except Exception:
            model_selector = None
        try:
            task_manager = app.query_one("#task-manager", TaskManagerWidget)
        except Exception:
            task_manager = None

        # 更新模型（若选择了）
        try:
            selected = model_selector.get_selected_model() if model_selector else None
            if selected:
                app.ai_client = app.ai_client.__class__(
                    system_prompt=getattr(app.ai_client, "system_prompt", ""),
                    model_name=selected,
                )
        except Exception:
            pass

        # 准备基础消息（优先使用应用的现有方法）
        try:
            base_messages = app._prepare_messages(original_user_text)
        except Exception:
            base_messages = [{"role": "user", "content": original_user_text or ""}]
        try:
            from .chat_stream import repair_openai_tool_sequence
            base_messages = repair_openai_tool_sequence(base_messages)
            base_messages, _ra, _rt = app._enforce_openai_tool_sequence(base_messages)
        except Exception:
            pass

        # 保存计划上下文，支持恢复
        try:
            setattr(app, "_current_plan_steps", list(steps or []))
            setattr(app, "_current_plan_task_text", original_user_text)
            setattr(app, "_current_plan_index", int(start_index or 0))
        except Exception as e:
            logger.warning(f"[state] 计划上下文保存异常: {e}")

        # 初始化任务管理器
        try:
            if task_manager:
                task_manager.set_tasks(steps)
                try:
                    si = int(start_index or 0)
                    for i in range(0, si):
                        task_manager.update_status(i + 1, "completed", 0)
                    if si < len(steps):
                        task_manager.update_status(si + 1, "in_progress", 0)
                except Exception:
                    pass
        except Exception:
            pass

        # 工具定义（新路径：通过 ai_helpers）
        try:
            enabled = getattr(app, "enabled_tools", set()) or set()
            tools = get_enabled_tools_openai_format(enabled)
        except Exception:
            tools = []

        # 初始化会话级记录容器（轻量）
        try:
            if not hasattr(app, "_session_step_records"):
                setattr(app, "_session_step_records", [])
            if not hasattr(app, "_session_tool_records"):
                setattr(app, "_session_tool_records", [])
        except Exception:
            pass

        # 步骤执行
        for idx in range(int(start_index or 0), len(steps)):
            step_text = steps[idx]
            try:
                setattr(app, "_current_plan_index", idx)
            except Exception:
                pass
            # 记录：步骤开始
            logger.info(f"[recv][step] 第 {idx+1} 步开始: {step_text}")

            step_record: Dict[str, Any] = {
                "index": idx + 1,
                "text": step_text,
                "iteration": 0,
                "status": "in_progress",
                "start_ts": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            }
            try:
                if chat_history:
                    chat_history.add_message("assistant", f"🔸 正在执行第 {idx+1} 步：{step_text}")
            except Exception:
                pass
            try:
                if task_manager:
                    task_manager.update_status(idx + 1, "in_progress", 0)
            except Exception:
                pass

            # 将当前步骤加入上下文并请求模型（允许自动工具）
            system_hint = {
                "role": "system",
                "content": f"从第 {int(start_index or 0)+1} 步开始执行。之前步骤视为已完成。当前步骤：{step_text}。仅按当前步骤执行，不要从第一步重新开始。"
            }
            user_turn = {"role": "user", "content": step_text}
            messages = list(base_messages) + [system_hint, user_turn]
            max_iterations = 8
            iteration = 0

            while iteration < max_iterations:
                iteration += 1
                step_record["iteration"] = iteration
                try:
                    if task_manager:
                        task_manager.update_status(idx + 1, "in_progress", iteration)
                except Exception:
                    pass

                # 若当前文本表征为不需要工具，仍然正常聊天；否则提示工具执行
                tool_choice = "auto"
                try:
                    if not needs_tool_call(step_text):
                        tool_choice = None
                except Exception:
                    tool_choice = "auto"

                try:
                    try:
                        from .chat_stream import repair_openai_tool_sequence
                        provider = getattr(app.ai_client.model_config, "provider", "")
                        messages = repair_openai_tool_sequence(messages)
                        messages = app._sanitize_tool_messages(messages, provider)
                        messages, _ra, _rt = app._enforce_openai_tool_sequence(messages)
                    except Exception:
                        pass
                    resp = await app.ai_client.chat_with_tools_async(
                        messages,
                        tools=tools,
                        tool_choice=tool_choice,
                    )
                except Exception as e:
                    safe_notify(app, f"模型调用异常：{e}", severity="error", timeout=4)
                    logger.warning(f"[state] 第 {idx+1} 步模型调用异常: {e}")
                    logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                    break

                assistant_text = getattr(resp, "content", "") or ""
                logger.info(f"[recv][resp] 第 {idx+1} 步回复: {resp}")
                tool_calls = _normalize_tool_calls(getattr(resp, "tool_calls", None))
                if tool_calls:
                    try:
                        for tc in tool_calls:
                            fn = (tc or {}).get("function", {})
                            name = fn.get("name") or ""
                            args = fn.get("arguments") or ""
                            logger.info(f"[tool] 第 {idx+1} 步调用: {name} (args_len={len(str(args))})")
                            # 记录工具调用（轻量摘要）
                            getattr(app, "_session_tool_records", []).append({
                                "step_index": idx + 1,
                                "name": name,
                                "args_preview": (str(args)[:200] if args else ""),
                                "ts": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                            })
                    except Exception:
                        pass

                # 展示助手消息
                try:
                    if chat_history:
                        chat_history.add_message("assistant", assistant_text)
                except Exception:
                    pass

                # 表单协议解析与弹窗（助手文本内出现表单块时立即触发）
                try:
                    form_schema = app.ai_client._parse_form_request_from_text(assistant_text or "")
                except Exception:
                    form_schema = None
                if form_schema:
                    try:
                        from ..widgets.modal_widgets import DynamicFormModal
                        app.push_screen(DynamicFormModal(form_schema))
                        logger.debug(f"[form] plan: pushed modal, fields={len((form_schema or {}).get('fields', []))}")
                    except Exception:
                        logger.error("[form] plan: push modal failed", exc_info=True)
                    try:
                        if task_manager:
                            task_manager.update_status(idx + 1, "paused", iteration)
                    except Exception:
                        pass
                    setattr(app, "plan_status", "paused")
                    step_record["status"] = "paused"
                    step_record["transition_reason"] = "助手文本包含表单块，触发弹窗并暂停"
                    logger.info(f"[transition] 第 {idx+1} 步 → PAUSE (表单弹窗)")
                    logger.info(f"[state] 会话状态=paused")
                    logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                    try:
                        getattr(app, "_session_step_records", []).append(step_record)
                    except Exception:
                        pass
                    try:
                        setattr(app, "_current_plan_index", idx)
                    except Exception:
                        pass
                    return

                # 早停与暂停判定（新标签）——仅当最后一步才允许 COMPLETE/DONE 直接结束
                is_final_step = (idx == len(steps) - 1)
                if _has_marker(assistant_text, NEW_TASK_DONE_MARKERS):
                    if is_final_step:
                        setattr(app, "plan_status", "completed")
                        try:
                            if task_manager:
                                task_manager.update_status(idx + 1, "completed", iteration)
                        except Exception:
                            pass
                        safe_notify(app, "✅ 任务已完成", timeout=3)
                        step_record["status"] = "completed"
                        step_record["transition_reason"] = "助手标记任务完成"
                        logger.info(f"[state] 会话状态=completed")
                        logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                        try:
                            getattr(app, "_session_step_records", []).append(step_record)
                        except Exception:
                            pass
                        break
                    else:
                        try:
                            if task_manager:
                                task_manager.update_status(idx + 1, "completed", iteration)
                        except Exception:
                            pass
                        step_record["status"] = "completed"
                        step_record["transition_reason"] = "非最后一步助手标记完成，推进下一步"
                        logger.info(f"[transition] 第 {idx+1} 步 → NEXT (助手标记)")
                        logger.info(f"[state] 会话状态=running")
                        logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                        try:
                            getattr(app, "_session_step_records", []).append(step_record)
                        except Exception:
                            pass
                        break
                if _has_marker(assistant_text, NEW_STEP_BLOCKED_MARKERS):
                    setattr(app, "plan_status", "paused")
                    safe_notify(app, "⛔ 当前步骤受阻，已暂停", severity="warning", timeout=4)
                    try:
                        if task_manager:
                            task_manager.update_status(idx + 1, "blocked", iteration)
                    except Exception:
                        pass
                    step_record["status"] = "blocked"
                    step_record["transition_reason"] = "助手标记受阻"
                    logger.info(f"[transition] 第 {idx+1} 步 → BLOCK")
                    logger.info(f"[state] 会话状态=paused")
                    logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                    try:
                        getattr(app, "_session_step_records", []).append(step_record)
                    except Exception:
                        pass
                    try:
                        setattr(app, "_current_plan_index", idx)
                    except Exception:
                        pass
                    return
                if _has_marker(assistant_text, NEW_STEP_WAIT_MARKERS):
                    setattr(app, "plan_status", "paused")
                    safe_notify(app, "⏸ 当前步骤需要你的确认或补充信息", severity="warning", timeout=4)
                    try:
                        if task_manager:
                            task_manager.update_status(idx + 1, "paused", iteration)
                    except Exception:
                        pass
                    step_record["status"] = "paused"
                    step_record["transition_reason"] = "助手标记需要用户确认/信息"
                    logger.info(f"[transition] 第 {idx+1} 步 → PAUSE")
                    logger.info(f"[state] 会话状态=paused")
                    logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                    try:
                        getattr(app, "_session_step_records", []).append(step_record)
                    except Exception:
                        pass
                    try:
                        setattr(app, "_current_plan_index", idx)
                    except Exception:
                        pass
                    return

                # RETRY 标签：继续迭代，达到上限则暂停
                if _has_marker(assistant_text, NEW_STEP_RETRY_MARKERS):
                    logger.info(f"[transition] 第 {idx+1} 步 → RETRY (迭代 {iteration}/{max_iterations})")
                    base_messages.append({"role": "assistant", "content": assistant_text})
                    if iteration >= max_iterations:
                        setattr(app, "plan_status", "paused")
                        try:
                            if task_manager:
                                task_manager.update_status(idx + 1, "paused", iteration)
                        except Exception:
                            pass
                        safe_notify(app, "⏸ 达到迭代上限，仍需重试，已暂停", severity="warning", timeout=4)
                        step_record["status"] = "paused"
                        step_record["transition_reason"] = "达到迭代上限，仍需重试"
                        logger.info(f"[state] 会话状态=paused")
                        logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                        try:
                            getattr(app, "_session_step_records", []).append(step_record)
                        except Exception:
                            pass
                        try:
                            setattr(app, "_current_plan_index", idx)
                        except Exception:
                            pass
                        return
                    messages = list(base_messages) + [system_hint, user_turn]
                    continue

                # 补充assistant消息中的工具调用以满足OpenAI工具序（如果存在）
                if tool_calls:
                    try:
                        base_messages.append({
                            "role": "assistant",
                            "content": assistant_text,
                            "tool_calls": tool_calls,
                        })
                    except Exception:
                        base_messages.append({"role": "assistant", "content": assistant_text})
                else:
                    base_messages.append({"role": "assistant", "content": assistant_text})

                # 执行工具并将结果加入上下文
                if tool_calls:
                    try:
                        tool_messages = await process_tool_calls(
                            tool_calls,
                            chat_history_widget=chat_history,
                            add_to_base_messages=True,
                        )
                        base_messages.extend(tool_messages or [])
                        logger.info(f"[tool] 第 {idx+1} 步工具结果消息数: {len(tool_messages or [])}")
                    except Exception as e:
                        safe_notify(app, f"工具调用异常：{e}", severity="warning", timeout=4)
                        logger.warning(f"[tool] 第 {idx+1} 步工具调用异常: {e}")

                # 总结与转移决定（阶段 4）
                try:
                    decision = await self._summarize_and_decide(app, base_messages, assistant_text)
                    tag = (decision.get("tag") or "").upper()
                    reason = str(decision.get("reason") or "").strip()
                    step_record["summary_tag"] = tag
                    if reason:
                        step_record["summary_reason"] = reason
                        try:
                            setattr(app, "_last_summary_reason", reason)
                        except Exception:
                            logger.warning(f"[summary] 第 {idx+1} 步设置总结原因异常: {e}")
                        
                        base_messages.append({"role": "system", "content": f"[总结原因]\n{reason}"})
                    logger.info(f"[summary] 第 {idx+1} 步判定标签: {tag}")
                except Exception:
                    tag = "NEXT"
                    step_record["summary_tag"] = tag

                if tag == "RETRY":
                    logger.info(f"[transition] 第 {idx+1} 步 → RETRY (总结判定，迭代 {iteration}/{max_iterations})")
                    if iteration >= max_iterations:
                        setattr(app, "plan_status", "paused")
                        try:
                            if task_manager:
                                task_manager.update_status(idx + 1, "paused", iteration)
                        except Exception:
                            pass
                        safe_notify(app, "⏸ 达到迭代上限，仍需重试，已暂停", severity="warning", timeout=4)
                        step_record["status"] = "paused"
                        step_record["transition_reason"] = "达到迭代上限，仍需重试（总结判定）"
                        logger.info(f"[state] 会话状态=paused")
                        logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                        try:
                            getattr(app, "_session_step_records", []).append(step_record)
                        except Exception:
                            pass
                        try:
                            setattr(app, "_current_plan_index", idx)
                        except Exception:
                            pass
                        return
                    messages = list(base_messages) + [system_hint, user_turn]
                    continue

                if tag in ("PAUSE", "WAIT"):
                    setattr(app, "plan_status", "paused")
                    try:
                        if task_manager:
                            task_manager.update_status(idx + 1, "paused", iteration)
                    except Exception:
                        pass
                    safe_notify(app, "⏸ 总结判定需要你的确认或补充信息", severity="warning", timeout=4)
                    try:
                        form_schema = app.ai_client._parse_form_request_from_text(assistant_text or "")
                    except Exception:
                        form_schema = None
                    if form_schema:
                        try:
                            from ..widgets.modal_widgets import DynamicFormModal
                            app.push_screen(DynamicFormModal(form_schema))
                            logger.debug(f"[form] plan: summary pushed modal, fields={len((form_schema or {}).get('fields', []))}")
                        except Exception:
                            logger.error("[form] plan: summary push modal failed", exc_info=True)
                    step_record["status"] = "paused"
                    step_record["transition_reason"] = "需要用户确认或补充信息"
                    logger.info(f"[transition] 第 {idx+1} 步 → PAUSE")
                    logger.info(f"[state] 会话状态=paused")
                    logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                    try:
                        getattr(app, "_session_step_records", []).append(step_record)
                    except Exception:
                        pass
                    return
                if tag in ("BLOCK", "BLOCKED"):
                    setattr(app, "plan_status", "paused")
                    try:
                        if task_manager:
                            task_manager.update_status(idx + 1, "blocked", iteration)
                    except Exception:
                        pass
                    safe_notify(app, "🛑 总结判定当前步骤被阻塞", severity="warning", timeout=4)
                    try:
                        reason_text = str(decision.get("reason") or getattr(app, "_last_summary_reason", "") or "").strip()
                        if base_messages and reason_text:
                            base_messages.append({"role": "assistant", "content": f"🛑 阻塞：{reason_text}\n请按提示处理后再点击“继续”或重试。"})
                    except Exception:
                        logger.error("[summary] 第 %d 步添加阻塞提示到聊天历史失败", idx+1, exc_info=True)
                        safe_notify(app, "🛑 总结判定当前步骤被阻塞，无法继续执行", severity="error", timeout=4)
                    step_record["status"] = "blocked"
                    step_record["transition_reason"] = "外部资源/权限受限或错误阻塞"
                    logger.info(f"[transition] 第 {idx+1} 步 → BLOCK")
                    logger.info(f"[state] 会话状态=paused")
                    logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                    try:
                        getattr(app, "_session_step_records", []).append(step_record)
                    except Exception:
                        pass
                    return
                if tag in ("COMPLETE", "DONE"):
                    if is_final_step:
                        setattr(app, "plan_status", "completed")
                        try:
                            if task_manager:
                                task_manager.update_status(idx + 1, "completed", iteration)
                        except Exception:
                            pass
                        safe_notify(app, "✅ 任务已完成（总结判定）", timeout=3)
                        step_record["status"] = "completed"
                        step_record["transition_reason"] = "总结判定完成"
                        logger.info(f"[transition] 第 {idx+1} 步 → COMPLETE")
                        logger.info(f"[state] 会话状态=completed")
                        logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                        try:
                            getattr(app, "_session_step_records", []).append(step_record)
                        except Exception:
                            pass
                        try:
                            setattr(app, "_current_plan_index", idx + 1)
                        except Exception:
                            pass
                        break
                    else:
                        try:
                            if task_manager:
                                task_manager.update_status(idx + 1, "completed", iteration)
                        except Exception:
                            pass
                        step_record["status"] = "completed"
                        step_record["transition_reason"] = "非最后一步按NEXT推进"
                        logger.info(f"[transition] 第 {idx+1} 步 → NEXT (非最后一步)")
                        logger.info(f"[state] 会话状态=running")
                        logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                        try:
                            getattr(app, "_session_step_records", []).append(step_record)
                        except Exception:
                            pass
                        try:
                            setattr(app, "_current_plan_index", idx + 1)
                        except Exception:
                            pass
                        break

                # 默认推进到下一步
                try:
                    if task_manager:
                        task_manager.update_status(idx + 1, "completed", iteration)
                except Exception:
                    pass
                step_record["status"] = "completed"
                step_record["transition_reason"] = "默认推进到下一步"
                logger.info(f"[transition] 第 {idx+1} 步 → NEXT")
                logger.info(f"[state] 会话状态=running")
                logger.info(f"步骤执行结束（第 {idx+1} 步，第 {iteration} 次迭代）")
                try:
                    getattr(app, "_session_step_records", []).append(step_record)
                except Exception:
                    pass
                try:
                    setattr(app, "_current_plan_index", idx + 1)
                except Exception:
                    pass
                break

        # 所有步骤结束后，若未显式置为完成，则置为完成
        try:
            if getattr(app, "plan_status", "running") != "paused":
                setattr(app, "plan_status", "completed")
                if chat_history:
                    chat_history.add_message("assistant", "✅ 任务步骤执行完成")
                logger.info("[state] 会话状态=completed")
                try:
                    setattr(app, "_current_plan_steps", [])
                    setattr(app, "_current_plan_index", 0)
                except Exception:
                    pass
        except Exception:
            pass

        # 执行完毕尝试保存会话（含步骤与工具记录）
        try:
            if hasattr(app, "_save_current_session"):
                app._save_current_session()
        except Exception:
            pass

    async def _summarize_and_decide(self, app: Any, messages: List[Dict[str, Any]], assistant_text: str) -> Dict[str, Any]:
        """对当前步骤进行总结并给出转移决定。

        返回格式示例：{"tag": "NEXT"}；可取值：NEXT、PAUSE、BLOCK、COMPLETE、DONE、WAIT、BLOCKED、RETRY。
        """

        prompt = """你是一个严格的步骤执行总结器。
- 仅基于最近的上下文消息（assistant 与 tool），并结合“当前步骤的目标”进行判定；。
- 最近消息选择：以最近一条非空的工具或助手消息为主，若两者同时存在则共同参考。
- 判定分场景：
  1) 文档/语法/规则检索类步骤（例如：检索 SPL 语法/规则、调用 get_docs、最近的工具/助手内容呈现文档结构：以标题开头、包含“核心原则/黄金模板/常见错误与修正/生成检查清单”等段落）：
     - 只要返回了非空且连贯的文档内容，并且未出现“错误/异常/未找到/空结果/黄色提示（如 [yellow] 或‘结果为空’）”，则判定为 NEXT。
     - 不要因为文档中出现“错误写法/错误示例/修正建议”等说明性内容而判定 RETRY（这些属于文档说明，并非工具执行失败）。
  2) 数据查询/执行类步骤（例如：检索日志/指标数据、执行统计/绘图等）：
     - 若最近消息明确出现“错误/异常/语法错误/修复建议/空结果/未查询到数据/提示放宽条件/黄色提示（如 [yellow] 或‘结果为空’）”，则返回 RETRY。下一轮应调整参数，避免与上一次完全相同（例如扩大时间范围、改用模糊匹配、减少过滤、必要时移除过严的 origin 过滤）。
     - 若无上述失败或空结果信号，则返回 NEXT。
- 标签使用：
  - 若任务整体完成，返回 COMPLETE（仅在最后一步使用）。
  - 若本步骤需要暂停，返回 PAUSE。
  - 若本步骤被阻塞，返回 BLOCKED。
  - 若本步骤已完成，返回 NEXT。
  - 若需要重试，返回 RETRY。
- 结合“当前任务目标（task_goal）”进行判定：若最近上下文已充分达成 task_goal，请返回 COMPLETE（仅最后一步）；若尚未达成，则根据本步情况返回 NEXT/RETRY/PAUSE/BLOCKED。
- 仅输出一个 JSON：{"tag": "NEXT|PAUSE|BLOCK|COMPLETE|RETRY", "reason": "判定原因"}。
- 不要添加其它额外字段。"""

        recent = messages[-20:] if messages and len(messages) > 20 else list(messages or [])
        task_goal = str(getattr(app, "current_task_goal", "") or "").strip()
        logger.info(f"[goal] summarize 判定阶段任务目标: {task_goal}")
        msgs = recent + [
            {"role": "system", "content": prompt},
            {"role": "assistant", "content": assistant_text or ""},
        ]
        if task_goal:
            msgs.append({"role": "user", "content": f"[当前任务目标]\n{task_goal}"})
        logger.info(f"[prompt] 执行总结器提示：{msgs}")
        try:
            resp = await app.ai_client.chat_async(msgs)
        except Exception:
            return {"tag": "NEXT"}
        logger.info(f"[response] 执行总结器响应：{resp}")
        try:
            text = getattr(resp, "content", "") or ""
            import json as _json
            obj = _json.loads(text)
            if not isinstance(obj, dict):
                raise ValueError("响应不是对象")
            return obj
        except Exception:
            return {"tag": "NEXT", "reason": "解析响应失败"}



from rich import console
from ..util import is_fuzzy_key
from .asset_map import get_resources
from ..base.client import ROOT_PATH
from ..converters import apply_field_converter

console = console.Console()


def _extract_field_aliases(default_fields):
    """从default_fields配置中提取字段别名映射
    
    Args:
        default_fields: 字段配置列表，可能包含字典格式的字段定义
        
    Returns:
        dict: 字段名到别名的映射字典
    """
    field_aliases = {}
    if not default_fields:
        return field_aliases
    
    for field_config in default_fields:
        if isinstance(field_config, dict):
            # 新格式：{"name": "field_name", "alias": "别名"}
            if "name" in field_config and "alias" in field_config:
                field_aliases[field_config["name"]] = field_config["alias"]
            elif "name" in field_config:
                # 只有name没有alias，使用字段名作为别名
                field_aliases[field_config["name"]] = field_config["name"]
        elif isinstance(field_config, str):
            # 处理字符串格式，保持原样
            field_aliases[field_config] = field_config
            
    return field_aliases


def _extract_field_converters(default_fields):
    """从default_fields配置中提取字段转换器配置
    
    Args:
        default_fields: 字段配置列表，可能包含字典格式的字段定义
        
    Returns:
        dict: 字段名到转换器配置的映射字典
    """
    field_converters = {}
    if not default_fields:
        return field_converters
    
    for field_config in default_fields:
        if isinstance(field_config, dict) and "name" in field_config and "convert" in field_config:
            field_converters[field_config["name"]] = field_config["convert"]
            
    return field_converters


def apply_field_conversions(data_list, field_converters):
    """对数据列表应用字段转换
    
    Args:
        data_list: 数据列表
        field_converters: 字段转换器配置字典
        
    Returns:
        list: 转换后的数据列表
    """
    if not data_list or not field_converters:
        return data_list
    
    converted_data = []
    for item in data_list:
        if isinstance(item, dict):
            converted_item = item.copy()
            for field_name, convert_config in field_converters.items():
                if field_name in converted_item:
                    original_value = converted_item[field_name]
                    converted_value = apply_field_converter(original_value, convert_config)
                    converted_item[field_name] = converted_value
            converted_data.append(converted_item)
        else:
            converted_data.append(item)
    
    return converted_data


def list_assets_request(asset_name, groupId=-1, order="desc", pageNo=1,
                        pageSize=10, prefix="", sort="updateTime", lang=None, **kwargs):
    path = asset_name
    query_fields = []
    ASSET_MAP = get_resources()
    key = is_fuzzy_key(asset_name, value_map=ASSET_MAP)
    default_fields = []
    fields_style = {}
    if key is not None:
        path = ASSET_MAP.get(key)["path"]
        if "methods" in ASSET_MAP.get(key) and "list" in ASSET_MAP.get(key)["methods"] and isinstance(ASSET_MAP.get(key)["methods"]['list'], dict):
            methods = ASSET_MAP.get(key)["methods"]
            path = methods.get('list', {}).get('path', path)
            default_fields = methods.get('list', {}).get('default_fields', [])
            fields_style = methods.get('list', {}).get('fields_style', [])
            if "query_fields" in methods['list']:
                query_fields = methods['list']["query_fields"]
        if not query_fields and "query_fields" in ASSET_MAP.get(key):
            query_fields = ASSET_MAP.get(key)["query_fields"]
        

    use_default_query_params = ASSET_MAP.get(key, {}).get("use_default_query_params", True)

    if use_default_query_params:
        query_params = {
            "groupId": groupId,
            "order": order,
            "pageNo": pageNo,
            "pageSize": pageSize,
            "prefix": prefix,
            "sort": sort,
        }
    else:
        query_params = {}
    for finfo in query_fields:
        f = finfo["field"]
        dft = finfo["default"]
        required = finfo["required"]
        if f in kwargs:
            query_params[f] = kwargs[f]
        elif required:
            query_params[f] = dft
    query_params.update(kwargs)

    custom_headers = {}
    if lang is not None and isinstance(lang, str):
        custom_headers["X-Pandora-Language"] = lang

    req_info = {
        "path": path,
        "query_params": query_params,
        # list 操作用不到的内容
        "method": "get",
        "data": {},
        "custom_headers": custom_headers,
        "fields_style": fields_style,
        "default_fields": default_fields,
        "field_aliases": _extract_field_aliases(default_fields),
        "field_converters": _extract_field_converters(default_fields),
    }
    # console.print(req_info)
    return req_info


def list_admin_request(asset_name, **kwargs):
    path = "/".join([ROOT_PATH, "admin/internal", asset_name])
    query_fields = []
    key = is_fuzzy_key(asset_name, value_map=ASSET_MAP)
    if key is not None:
        path = ASSET_MAP.get(key)["path"]
        if "query_fields" in ASSET_MAP.get(key):
            query_fields = ASSET_MAP.get(key)["query_fields"]

    query_params = {"format": "json"}
    for finfo in query_fields:
        f = finfo["field"]
        dft = finfo["default"]
        required = finfo["required"]
        if f in kwargs:
            query_params[f] = kwargs[f]
        elif required:
            query_params[f] = dft

    custom_headers = {}

    return {
        "path": path,
        "query_params": query_params,
        # list 操作用不到的内容
        "method": "get",
        "data": {},
        "custom_headers": custom_headers,
    }



"""
资产管理相关命令模块
"""
import json
import sys
import time
from mando import command, arg
from rich.console import Console
from rich.live import Live
from ketacli.sdk.base.client import request_get, request, download_file
from ketacli.sdk.request.list import list_assets_request, list_admin_request
from ketacli.sdk.request.get import get_asset_by_id_request
from ketacli.sdk.request.create import create_asset_request
from ketacli.sdk.request.update import update_asset_request
from ketacli.sdk.request.delete import delete_asset_request
from ketacli.sdk.request.export import export_asset_request
from ketacli.sdk.request.asset_map import get_resources
from ketacli.sdk.output.output import list_output, describe_output, get_asset_output
from ketacli.sdk.output.format import format_table
from ketacli.sdk.util import parse_url_params
from ketacli.sdk.test.cli import test_command

console = Console(markup=False)


@command('list')
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def _list(asset_type, groupId=-1, order="desc", pageNo=1, pageSize=10, prefix="", sort="updateTime", fields="",
          format=None, raw=False, lang=None, extra=None, watch=False, interval=3.0, show_all_fields=False, test=False):
    """List asset (such as repo,sourcetype,metric...) from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -l, --pageSize: Limit the page size.
    :param --pageNo: Limit the page number.
    :param --prefix: Fuzzy query filter.
    :param --sort: The field used to order by
    :param --order: The sort order, desc|asc
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --groupId: The resource group id.
    :param --raw: Prettify the time field or output the raw timestamp, if specified, output the raw format
    :param --lang: Choose the language preference of return value
    :param -e, --extra: extra query filter, example: include_defaults=true,flat_settings=true
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    :param -a, --show_all_fields: all fields to display
    :param --test: Enable test mode for assertion control
    """
    extra_dict = {}
    if extra is not None:
        # 解析 url 参数为 dict
        extra_dict = parse_url_params(extra)

    def generate_table():
        req = list_assets_request(
            asset_type, groupId, order, pageNo, pageSize, prefix, sort, lang, **extra_dict)
        resp = request_get(req["path"], req["query_params"],
                           req["custom_headers"]).json()
        output_fields = req.get("default_fields", [])
        field_aliases = req.get("field_aliases", {})
        field_converters = req.get("field_converters", {})
        if show_all_fields:
            output_fields = []
        if len(fields.strip()) > 0:
            output_fields = fields.strip().split(",")
        table = list_output(asset_type, output_fields=output_fields, resp=resp, field_aliases=field_aliases, field_converters=field_converters)
        if not table:
            return None
        return format_table(table, format, not raw)

    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "list"
        args.params = {"groupId": groupId, "order": order, "pageNo": pageNo, "pageSize": pageSize, 
                      "prefix": prefix, "sort": sort, "fields": fields, "lang": lang}
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()
    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any {asset_type}")
        else:
            console.print(table, overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
def admin(asset_type, format="json", extra=None, watch=False, interval=3.0):
    """List asset (such as repo,sourcetype,metric...) from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -f, --format: The output format, text|json|csv|html|latex
    :param -e, --extra: extra query filter, example: include_defaults=true,flat_settings=true
    :param -w, --watch: Watch the resource change
    :param --interval: refresh the resource change
    """
    extra_dict = {}
    if extra is not None:
        # 解析 url 参数为 dict
        extra_dict = parse_url_params(extra)

    def generate_table():
        req = list_admin_request(asset_type, **extra_dict)
        resp = request_get(req["path"], req["query_params"], req["custom_headers"]).json()
        output_fields = []
        table = list_output(asset_type, output_fields=output_fields, resp=resp)
        return format_table(table, format)

    if watch:
        with Live(generate_table(), console=console, refresh_per_second=1) as live:
            while True:
                try:
                    table = generate_table()
                    live.update(table)
                    time.sleep(interval)
                except KeyboardInterrupt:
                    live.stop()
                    sys.exit()
    else:
        table = generate_table()
        if table is None:
            console.print(f"we cannot find any {asset_type}")
        else:
            console.print(table)


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def get(asset_type, asset_id, fields="", format=None, lang=None, extra=None):
    """Get asset detail info from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param asset_id: The unique id of asset. (such as id or name...)
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --lang: Choose the language preference of return value
    :param -e, --extra: extra args, example:id=1234567890,name=test
    """
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    extra_args_map['name'] = asset_id

    req = get_asset_by_id_request(
        asset_type=asset_type, asset_id=asset_id, lang=lang, **extra_args_map)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    if format == "json":
        console.print(json.dumps(resp, indent=2, ensure_ascii=False), markup=False)
        return

    output_fields = []
    if len(fields.strip()) > 0:
        output_fields = fields.strip().split(",")
    table = get_asset_output(output_fields=output_fields, resp=resp)
    table.align = "l"
    if table is None:
        console.print(f"we cannot find any {asset_type}")
    else:
        console.print(format_table(table, format), overflow="fold")


@command
@arg("format", type=str,
     completer=lambda prefix, **kwd: [x for x in ["table", "text", "json", "csv", "html", "latex"] if
                                      x.startswith(prefix)])
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def describe(asset_type, format=None):
    """Describe the schema of asset type

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -f, --format: The output format, text|json|csv|html|latex
    """
    req = list_assets_request(asset_type)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    table = describe_output(asset_type, resp=resp)
    if table is None:
        console.print(f"we cannot find any {asset_type}")
    else:
        console.print(format_table(table, format), overflow="fold")


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def create(asset_type, name=None, data=None, file=None, extra=None, test=False):
    """Create asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name
    :param --data: The json string data {...}
    :param --file: Upload json text from file path.
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --test: Enable test mode for assertion control
    """
    if data is None and file is None:
        content = {}
    else:
        content = data
        if file is not None:
            f = open(file, encoding="utf-8")
            content = f.read()
        try:
            content = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    for k, v in extra_args_map.items():
        if str(v).startswith('@'):
            c = open(v[1:], encoding="utf-8").read().replace("\n", "\\n").replace("\"", "\\\"")
            extra_args_map[k] = c
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')
    
    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "create"
        args.params = {"name": name, "data": content}
        if extra_args_map:
            args.params.update(extra_args_map)
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return

    try:
        req = create_asset_request(asset_type, name, content, **extra_args_map)
        resp = request(req["method"], req["path"], data=req['data']).json()
        console.print(json.dumps(resp, ensure_ascii=False), overflow="fold", markup=False)
    except Exception as e:
        console.print(f"create asset {name} failed, error: {e}")


def get_operation_type(prefix, **kwargs):
    operators = []
    for x in get_resources():
        if x == kwargs.get('parsed_args').asset_type:
            methods = get_resources()[x].get('methods')
            operators = methods.keys()
    operators = [x for x in operators if
                 x not in ['list', 'create', 'update', 'delete', 'download', 'get'] and x.startswith(prefix)]
    return operators


@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
@command
@arg("operation", type=str, completer=get_operation_type)
def update(asset_type, name=None, operation="update", data=None, file=None, extra=None, test=False):
    """Update asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name
    :param -d, --data: The json string data {...}
    :param -f, --file: Upload json text from file path.
    :param -o, --operation: operation type, such as open, close, update, delete
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --test: Enable test mode for assertion control
    """
    if data is None and file is None:
        data = {}
    else:
        content = data
        if file is not None:
            f = open(file, encoding="utf-8")
            content = f.read()

        try:
            data = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')
    
    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "update"
        args.params = {"name": name, "operation": operation, "data": data}
        if extra_args_map:
            args.params.update(extra_args_map)
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return
    
    req = update_asset_request(asset_type, operation, name, data, **extra_args_map)
    resp = request(req["method"], req["path"], data=req['data']).json()
    console.print(resp, overflow="fold")


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def download(asset_type, extra=None, base_path="./"):
    """export file asset

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --base_path: the file save path
    """
    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}

    req = export_asset_request(asset_type, **extra_args_map)
    download_file(req["path"], save_path=base_path)


@command
@arg("asset_type", type=str,
     completer=lambda prefix, **kwd: [x for x in get_resources().keys() if x.startswith(prefix)])
def delete(asset_type, name=None, data=None, file=None, extra=None, test=False):
    """Delete asset

    :param --asset_type: The target asset type, such as repo, sourcetype ...
    :param -n, --name: The target asset name or id
    :param -d, --data: The json string data {...}
    :param -f, --file: Upload json text from file path.
    :param -e, --extra: extra args, example:id=1234567890,name=test
    :param --test: Enable test mode for assertion control
    """
    if data is None and file is None:
        data = {}
    else:
        content = data
        if file is not None:
            f = open(file, encoding="utf8")
            content = f.read()

        try:
            data = json.loads(content)
        except json.JSONDecodeError as e:
            console.print("JSON 解析错误:", e)
            return

    if extra:
        extra_args_map = parse_url_params(extra)
    else:
        extra_args_map = {}
    if 'name' in extra_args_map:
        name = extra_args_map.pop('name')

    # 测试模式逻辑
    if test:
        from types import SimpleNamespace
        args = SimpleNamespace()
        args.action = "run"
        args.asset_type = asset_type
        args.method = "delete"
        args.params = {"name": name, "data": data}
        if extra_args_map:
            args.params.update(extra_args_map)
        args.suite_file = None
        args.config_file = None
        args.format = "console"
        args.output = None
        test_command(args)
        return

    req = delete_asset_request(asset_type, name, data, **extra_args_map)
    resp = request(req["method"], req["path"], data=req['data']).json()
    console.print(resp, overflow="fold")


"""消息显示组件

包含各种消息显示相关的UI组件。
"""

import pyperclip
import uuid
from typing import Dict, List, Optional
from datetime import datetime

from textual.app import ComposeResult
from textual.containers import Container
from textual.widgets import Static, Button, Markdown
from textual.widget import Widget
from textual.containers import Container
import logging
from textual.reactive import reactive
from textual.widget import Widget
from .message_actions_widget import MessageActions

from ..token_calculator import TokenStats, calculate_token_stats
from typing import Optional, Dict


def safe_markdown_widget(content: str, **kwargs):
    """安全地创建Markdown组件，如果解析失败则回退到Static组件"""
    try:
        return Markdown(content, **kwargs)
    except Exception:
        # 如果Markdown解析失败，回退到Static组件
        return Static(content, **kwargs)


class ExpandableContentMixin:
    """抽象可展开的文本内容逻辑：统一展开/收起、内容更新与复制"""

    def _render_actions(self, is_long: bool, on_copy):
        return MessageActions(
            on_toggle=self._on_actions_toggle,
            on_copy=on_copy,
            is_long=is_long,
            is_expanded=self.is_expanded,
            unique_id=self.unique_id,
            id=f"message-actions-{self.unique_id}",
        )

    def _update_text_content(
        self,
        text: str,
        is_markdown: bool,
        content_id: str,
        max_length: int,
        prefix: str = "",
    ):
        """统一更新文本内容组件（支持Markdown/Static），并在必要时挂载新组件"""
        try:
            content_widget = self.query_one(
                f"#{content_id}", Markdown if is_markdown else Static
            )
        except Exception:
            content_widget = None

        display_text = prefix + (
            text[:max_length] + "..." if len(text) > max_length and not self.is_expanded else text
        )

        if content_widget:
            content_widget.update(display_text)
        else:
            try:
                anchor = self.query_one(f"#message-actions-{self.unique_id}")
            except Exception:
                anchor = None
            if is_markdown:
                new_widget = safe_markdown_widget(
                    display_text, classes="message-content", id=content_id
                )
            else:
                new_widget = Static(
                    display_text, classes="message-content", id=content_id, markup=False
                )
            if anchor:
                self.mount(new_widget, before=anchor)
            else:
                self.mount(new_widget)

    def _copy_text_to_clipboard(self, text: str):
        try:
            pyperclip.copy(text or "")
            # 复制成功提示
            if hasattr(self, "notify"):
                self.notify("内容已复制到剪贴板", severity="success")
            elif hasattr(self.app, "notify"):
                self.app.notify("内容已复制到剪贴板", severity="success")
        except Exception as e:
            if hasattr(self, "notify"):
                self.notify(f"复制失败: {str(e)}", severity="error")
            elif hasattr(self.app, "notify"):
                self.app.notify(f"复制失败: {str(e)}", severity="error")


class MessageWidget(ExpandableContentMixin, Static):
    """单条消息显示组件"""
    
    def __init__(self, role: str, content: str, timestamp: str = None, 
                 token_stats: Optional[TokenStats] = None, 
                 context_messages: Optional[List[Dict]] = None, **kwargs):
        super().__init__(markup=False, **kwargs)
        self.role = role
        self.content = content
        self.timestamp = timestamp or datetime.now().strftime("%H:%M:%S")
        self.is_expanded = False
        # 为每个实例生成唯一ID前缀，避免多个实例间的ID冲突
        self.unique_id = str(uuid.uuid4())[:8]
        
        # Token统计相关
        self.context_messages = context_messages or []
        
        # 如果没有提供token统计，则自动计算
        if token_stats is None:
            message_dict = {"role": role, "content": content}
            self.token_stats = calculate_token_stats(
                current_message=message_dict,
                context_messages=self.context_messages
            )
        else:
            self.token_stats = token_stats
        
        
    def compose(self) -> ComposeResult:
        """构建消息UI"""
        role_color = "blue" if self.role == "user" else "green"
        role_text = "用户" if self.role == "user" else "AI助手"
        
        with Container(classes=f"message-container {self.role}-message"):
            yield Static(f"[{role_color}]{role_text}[/{role_color}] [{self.timestamp}]", classes="message-header")
            
            # 检查内容长度
            is_long = len(self.content) > 500

            # 内容展示（统一逻辑：根据 is_expanded 状态截断/展示）
            if self.role == "assistant":
                if is_long and not self.is_expanded:
                    truncated_content = self.content[:500] + "..."
                    yield safe_markdown_widget(truncated_content, classes="message-content", id=f"message-content-{self.unique_id}")
                else:
                    yield safe_markdown_widget(self.content, classes="message-content", id=f"message-content-{self.unique_id}")
            else:
                if is_long and not self.is_expanded:
                    truncated_content = self.content[:500] + "..."
                    yield Static(truncated_content, classes="message-content", id=f"message-content-{self.unique_id}", markup=False)
                else:
                    yield Static(self.content, classes="message-content", id=f"message-content-{self.unique_id}", markup=False)

            # 操作组件：复制 + 展开/收起（统一）
            yield self._render_actions(is_long, on_copy=self._copy_message_content)

            # 添加token统计显示
            yield self._create_token_stats_widget()
    
    def on_button_pressed(self, event: Button.Pressed) -> None:
        """消息按钮事件已统一由 MessageActions 处理"""
        pass

    def _on_actions_toggle(self, expanded: bool):
        """供 MessageActions 回调：切换展开状态"""
        self.is_expanded = expanded
        self._update_message_content()
    
    def _update_message_content(self):
        """更新消息内容显示（仅更新内容，操作按钮由 MessageActions 管理）"""
        is_markdown = self.role == "assistant"
        self._update_text_content(
            text=self.content,
            is_markdown=is_markdown,
            content_id=f"message-content-{self.unique_id}",
            max_length=500,
            prefix="",
        )

    def _copy_message_content(self):
        """复制消息内容到剪贴板"""
        self._copy_text_to_clipboard(self.content)

    def _create_token_stats_widget(self) -> Static:
        """创建token统计显示组件"""
        stats_text = f"[dim]🔢 {str(self.token_stats)}[/dim]"
        return Static(stats_text, classes="token-stats", id=f"token-stats-{self.unique_id}")

    def update_token_stats(self, new_stats: TokenStats):
        """更新token统计信息"""
        self.token_stats = new_stats
        try:
            stats_widget = self.query_one(f"#token-stats-{self.unique_id}", Static)
            stats_text = f"[dim]🔢 {str(new_stats)}[/dim]"
            stats_widget.update(stats_text)
        except Exception:
            # 如果组件不存在，重新创建
            self.mount(self._create_token_stats_widget())


class StreamingMessageWidget(Static):
    """流式消息显示组件"""
    
    def __init__(self, role: str, timestamp: str, 
                 context_messages: Optional[List[Dict]] = None, **kwargs):
        super().__init__(markup=False, **kwargs)
        self.role = role
        self.timestamp = timestamp
        self.content_chunks = []
        # 为每个实例生成唯一ID前缀，避免多个实例间的ID冲突
        self.unique_id = str(uuid.uuid4())[:8]
        
        # Token统计相关
        self.context_messages = context_messages or []
        self.token_stats = None
        
    def compose(self) -> ComposeResult:
        """构建流式消息UI"""
        role_color = "blue" if self.role == "user" else "green"
        role_text = "用户" if self.role == "user" else "AI助手"
        
        with Container(classes=f"message-container {self.role}-message"):
            yield Static(f"[{role_color}]{role_text}[/{role_color}] [{self.timestamp}] [dim]正在输入...[/dim]", classes="message-header")
            # 使用Markdown进行流式渲染，保持Markdown格式
            yield safe_markdown_widget("", id=f"streaming-content-{self.unique_id}", classes="message-content")
            
    def append_content(self, chunk: str):
        """追加内容块"""
        self.content_chunks.append(chunk)
        current_content = "".join(self.content_chunks)
        
        # 更新内容（优先使用Markdown组件）
        try:
            content_widget = self.query_one(f"#streaming-content-{self.unique_id}", Markdown)
        except Exception:
            content_widget = self.query_one(f"#streaming-content-{self.unique_id}", Static)
        content_widget.update(current_content)
        
        # 滚动到底部
        if self.parent:
            self.parent.scroll_end()
            
    def finalize_content(self):
        """完成内容输入，移除"正在输入"提示"""
        header_widget = self.query_one(".message-header", Static)
        role_color = "blue" if self.role == "user" else "green"
        role_text = "用户" if self.role == "user" else "AI助手"
        header_widget.update(f"[{role_color}]{role_text}[/{role_color}] [{self.timestamp}]")
        
    def finalize(self):
        """完成流式消息，添加复制按钮和token统计"""
        # 计算最终的token统计
        final_content = "".join(self.content_chunks)
        message_dict = {"role": self.role, "content": final_content}
        self.token_stats = calculate_token_stats(
            current_message=message_dict,
            context_messages=self.context_messages
        )
        
        # 添加复制按钮
        copy_button = Button("📋 复制", id=f"copy-streaming-button-{self.unique_id}", classes="copy-button")
        self.mount(copy_button)
        
        # 添加token统计显示
        stats_text = f"[dim]🔢 {str(self.token_stats)}[/dim]"
        token_stats_widget = Static(stats_text, classes="token-stats", id=f"token-stats-{self.unique_id}")
        self.mount(token_stats_widget)
        
    def on_button_pressed(self, event: Button.Pressed) -> None:
        """处理按钮点击事件"""
        if event.button.id == f"copy-streaming-button-{self.unique_id}":
            self._copy_message_content()
    
    def _copy_message_content(self):
        """复制消息内容到剪贴板"""
        try:
            content = "".join(self.content_chunks)
            pyperclip.copy(content)
            if hasattr(self, "notify"):
                self.notify("内容已复制到剪贴板", severity="success")
            elif hasattr(self.app, "notify"):
                self.app.notify("内容已复制到剪贴板", severity="success")
        except Exception as e:
            if hasattr(self, "notify"):
                self.notify(f"复制失败: {str(e)}", severity="error")
            elif hasattr(self.app, 'notify'):
                self.app.notify(f"复制失败: {str(e)}", severity="error")
        
    def get_final_content(self) -> str:
        """获取最终内容"""
        return "".join(self.content_chunks)


class ToolCallWidget(ExpandableContentMixin, Static):
    """工具调用显示组件"""
    
    def __init__(self, tool_name: str, arguments: str, result: str = None, success: bool = True, result_obj: Optional[Dict] = None, **kwargs):
        super().__init__(markup=False, **kwargs)
        self.tool_name = tool_name
        self.arguments = arguments
        self.result = result
        self.success = success
        self.result_obj = result_obj or None
        self.is_expanded = False
        # 为每个实例生成唯一ID前缀，避免多个实例间的ID冲突
        self.unique_id = str(uuid.uuid4())[:8]
        self.max_length = 500
        self._logger = logging.getLogger("ketacli.textual.tool_widget")
        
    def compose(self) -> ComposeResult:
        """构建工具调用UI"""
        status_text = "✓" if self.success else "✗"
        status_class = "success" if self.success else "error"
        self._logger.debug(f"[tool.ui] compose name={self.tool_name} has_obj={isinstance(self.result_obj, Widget)} result_is_widget={isinstance(self.result, Widget)}")

        # 将结果与操作按钮一并置于工具容器（黄色框）内
        with Container(classes="tool-call-container"):
            # 使用纯文本，避免用户输入中的 [] 被解析为富文本标记
            yield Static(
                f"🔧 调用工具: {self.tool_name} {status_text}",
                classes=f"tool-header {status_class}",
                id=f"tool-header-{self.unique_id}",
                markup=False,
            )
            yield Static(f"参数: {self.arguments}", classes="tool-args", markup=False)
            yield Button("✎ 编辑参数", id=f"edit-args-{self.unique_id}", classes="edit-args")

            # 先渲染文本结果（如有）
            if self.result:
                # 文本结果显示（统一逻辑：按字符长度截断）
                is_long = len(self.result) > 500
                if is_long and not self.is_expanded:
                    truncated = self.result[:500] + "..."
                    yield Static(
                        f"结果: {truncated}",
                        classes="tool-result",
                        id=f"tool-result-content-{self.unique_id}",
                        markup=False,
                    )
                else:
                    yield Static(
                        f"结果: {self.result}",
                        classes="tool-result",
                        id=f"tool-result-content-{self.unique_id}",
                        markup=False,
                    )

                # 操作组件：复制 + 展开/收起（统一）放在容器内部（工具消息固定显示展开按钮）
                yield self._render_actions(True, on_copy=self._copy_result_content)

            # 始终渲染一个结果容器，便于后续挂载可视化组件
            result_wrapper = Container(classes="tool-result-wrapper", id=f"tool-result-wrapper-{self.unique_id}")
            if self.result_obj and isinstance(self.result_obj, Widget):
                try:
                    self.notify("图表可视化结果已显示")
                except Exception:
                    pass
                result_wrapper.mount(self.result_obj)
            elif isinstance(self.result, Widget):
                try:
                    self.notify("图表可视化结果已显示")
                except Exception:
                    pass
                result_wrapper.mount(self.result)
            yield result_wrapper

    
    def on_button_pressed(self, event: Button.Pressed) -> None:
        if event.button.id == f"edit-args-{self.unique_id}":
            import json as _json
            from ..widgets.modal_widgets import ToolArgsEditDialog
            try:
                args = self.arguments
                if isinstance(args, str):
                    aobj = _json.loads(args)
                else:
                    aobj = args if isinstance(args, dict) else {}
            except Exception:
                aobj = args
            try:
                if hasattr(self.app, "push_screen"):
                    self.app.push_screen(ToolArgsEditDialog(tool_name=self.tool_name, original_args=aobj, error_summary=""))
            except Exception:
                pass

    def _on_actions_toggle(self, expanded: bool):
        """供 MessageActions 回调：切换展开状态"""
        self.is_expanded = expanded
        self._update_content()

    def _copy_result_content(self):
        """复制工具结果到剪贴板"""
        self._copy_text_to_clipboard(self.result or "")
    
    def _ensure_actions(self):
        from textual.containers import Container
        from textual.widgets import Button
        from .message_actions_widget import MessageActions
        actions_id = f"#message-actions-{self.unique_id}"
        try:
            actions_container = self.query_one(actions_id, MessageActions)
            return
        except Exception:
            actions_container = None
        # 工具消息固定显示展开按钮
        actions = self._render_actions(True, on_copy=self._copy_result_content)
        try:
            container = self.query_one(".tool-call-container", Container)
            container.mount(actions)
        except Exception:
            self.mount(actions)

    def _update_content(self):
        """更新内容显示"""
        if not self.result:
            return
        self._update_text_content(
            text=self.result,
            is_markdown=False,
            content_id=f"tool-result-content-{self.unique_id}",
            max_length=self.max_length,
            prefix="结果: ",
        )
        self._ensure_actions()

        # 优先就地更新现有结果组件，避免影响操作条
        try:
            content_widget = self.query_one(
                f"#tool-result-content-{self.unique_id}", Static
            )
        except Exception:
            content_widget = None

        

        if self.result:
            is_long = len(self.result) > self.max_length
            if content_widget:
                if is_long and not self.is_expanded:
                    truncated = self.result[:self.max_length] + "..."
                    content_widget.update(f"结果: {truncated}")
                else:
                    content_widget.update(f"结果: {self.result}")
            else:
                # 若不存在结果组件，则创建；为保证位置稳定，尽量插入到操作条之前
                try:
                    anchor = self.query_one(f"#message-actions-{self.unique_id}")
                except Exception:
                    anchor = None
                if is_long and not self.is_expanded:
                    truncated = self.result[:self.max_length] + "..."
                    new_widget = Static(
                        f"结果: {truncated}",
                        classes="tool-result",
                        id=f"tool-result-content-{self.unique_id}",
                        markup=False,
                    )
                else:
                    new_widget = Static(
                        f"结果: {self.result}",
                        classes="tool-result",
                        id=f"tool-result-content-{self.unique_id}",
                        markup=False,
                    )
                if anchor:
                    self.mount(new_widget, before=anchor)
                else:
                    self.mount(new_widget)

        
    def update_result(self, result: str, success: bool, result_obj: Optional[Widget] = None):
        """更新工具结果与状态"""
        self.result = result
        self.success = bool(success)
        if result_obj is not None:
            self.result_obj = result_obj
        self._logger.debug(f"[tool.ui] update_result name={self.tool_name} success={self.success} has_obj={isinstance(self.result_obj, Widget)} result_len={len(self.result or '')}")
        # 更新头部状态图标
        try:
            header = self.query_one(f"#tool-header-{self.unique_id}", Static)
            status_text = "✓" if self.success else "✗"
            header.update(f"🔧 调用工具: {self.tool_name} {status_text}")
        except Exception:
            pass
        # 更新结果显示
        if self.result_obj and isinstance(self.result_obj, Widget):
            wrapper = None
            try:
                wrapper = self.query_one(f"#tool-result-wrapper-{self.unique_id}", Container)
            except Exception:
                wrapper = None
            self._logger.debug(f"[tool.ui] wrapper_found={wrapper is not None}")
            if wrapper:
                self._logger.debug(f"[tool.ui] mount_into_existing_wrapper_start")
                for child in list(wrapper.children):
                    child.remove()
                wrapper.mount(self.result_obj)
                self._logger.debug(f"[tool.ui] mount_into_existing_wrapper_done children={len(list(wrapper.children))}")
            else:
                new_wrapper = Container(classes="tool-result-wrapper", id=f"tool-result-wrapper-{self.unique_id}")
                self._logger.debug(f"[tool.ui] create_wrapper_and_mount_start")
                new_wrapper.mount(self.result_obj)
                self.mount(new_wrapper)
                self._logger.debug(f"[tool.ui] create_wrapper_and_mount_done children={len(list(new_wrapper.children))}")
            self.refresh(layout=True)
            try:
                self.result_obj.refresh()
            except Exception:
                pass
        else:
            self._update_content()

        



"""
Function Call 功能模块

提供AI大模型的function call功能，支持动态注册和调用ketacli的各种功能。
"""

import json
import inspect
import asyncio
import os
from concurrent.futures import ThreadPoolExecutor
from typing import Dict, List, Any, Callable, Optional
from dataclasses import dataclass
from rich.console import Console
import logging

console = Console(markup=False)
logger = logging.getLogger("ketacli.textual")
tool_logger = logging.getLogger("ketacli.toolcalls")
if not tool_logger.handlers:
    tool_logger.setLevel(logging.DEBUG)
    try:
        base_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "../../.."))
        log_dir = os.path.join(base_dir, "log")
        os.makedirs(log_dir, exist_ok=True)
        log_path = os.path.join(log_dir, "tool_calls.log")
        fh = logging.FileHandler(log_path)
        fh.setFormatter(logging.Formatter("%(asctime)s [%(levelname)s] %(message)s"))
        tool_logger.addHandler(fh)
    except Exception:
        pass

def _preview_text(value: Any, limit: int = 1000) -> str:
    try:
        if value is None:
            return ""
        if isinstance(value, bytes):
            try:
                value = value.decode("utf-8", errors="replace")
            except Exception:
                value = str(value)
        if isinstance(value, (dict, list)):
            try:
                text = json.dumps(value, ensure_ascii=False)[:limit]
                return text
            except Exception:
                value = str(value)
        text = str(value)
        if len(text) > limit:
            return text[:limit] + "..."
        return text
    except Exception:
        return ""


@dataclass
class FunctionSchema:
    """函数模式定义"""
    name: str
    description: str
    parameters: Dict[str, Any]
    function: Callable


class FunctionRegistry:
    """函数注册表"""
    
    def __init__(self):
        self._functions: Dict[str, FunctionSchema] = {}
    
    def register(self, name: str, description: str, parameters: Dict[str, Any] = None):
        """注册函数装饰器"""
        def decorator(func: Callable):
            if parameters is None:
                # 自动从函数签名生成参数模式
                sig = inspect.signature(func)
                auto_params = self._generate_parameters_from_signature(sig)
            else:
                auto_params = parameters
            
            schema = FunctionSchema(
                name=name,
                description=description,
                parameters=auto_params,
                function=func
            )
            self._functions[name] = schema
            return func
        return decorator
    
    def _generate_parameters_from_signature(self, sig: inspect.Signature) -> Dict[str, Any]:
        """从函数签名自动生成参数模式"""
        properties = {}
        required = []
        
        for param_name, param in sig.parameters.items():
            if param_name.startswith('_'):  # 跳过私有参数
                continue
                
            param_info = {
                "type": "string"  # 默认类型
            }
            
            # 根据类型注解推断参数类型
            if param.annotation != inspect.Parameter.empty:
                if param.annotation == int:
                    param_info["type"] = "integer"
                elif param.annotation == float:
                    param_info["type"] = "number"
                elif param.annotation == bool:
                    param_info["type"] = "boolean"
                elif param.annotation == list:
                    param_info["type"] = "array"
                elif param.annotation == dict:
                    param_info["type"] = "object"
            
            # 如果没有默认值，则为必需参数
            if param.default == inspect.Parameter.empty:
                required.append(param_name)
            else:
                param_info["default"] = param.default
            
            properties[param_name] = param_info
        
        return {
            "type": "object",
            "properties": properties,
            "required": required
        }
    
    def get_function(self, name: str) -> Optional[FunctionSchema]:
        """获取注册的函数"""
        return self._functions.get(name)
    
    def list_functions(self) -> List[FunctionSchema]:
        """列出所有注册的函数"""
        return list(self._functions.values())
    
    def get_openai_tools_format(self) -> List[Dict[str, Any]]:
        """获取OpenAI tools格式的函数定义"""
        tools = []
        for func_schema in self._functions.values():
            tool = {
                "type": "function",
                "function": {
                    "name": func_schema.name,
                    "description": func_schema.description,
                    "parameters": func_schema.parameters
                }
            }
            tools.append(tool)
        return tools


class FunctionCallExecutor:
    """函数调用执行器"""
    
    def __init__(self, registry: FunctionRegistry):
        self.registry = registry
        self.executor = ThreadPoolExecutor(max_workers=4)  # 线程池用于异步执行
        # 设置线程池为守护模式，这样主线程退出时不会阻塞
        import atexit
        atexit.register(self.shutdown)
    
    def shutdown(self):
        """关闭线程池，确保资源释放"""
        if hasattr(self, 'executor') and self.executor:
            self.executor.shutdown(wait=False)
    
    def execute(self, function_name: str, arguments: Dict[str, Any]) -> Any:
        """执行函数调用"""
        func_schema = self.registry.get_function(function_name)
        if not func_schema:
            raise ValueError(f"未找到函数: {function_name}")
        
        try:
            logger.debug(f"[fc.sync] 调用函数: {function_name} args={arguments}")
            tool_logger.info(f"[start] fn={function_name} args={_preview_text(arguments)}")
            # 执行函数
            result = func_schema.function(**arguments)
            tool_logger.info(
                f"[success] fn={function_name} type={type(result).__name__} preview={_preview_text(result)}"
            )
            return {
                "success": True,
                "result": result,
                "function_name": function_name
            }
        except Exception as e:
            console.print(f"[red]执行函数 {function_name} 时出错: {e}[/red]")
            tool_logger.error(f"[error] fn={function_name} err={str(e)}")
            return {
                "success": False,
                "error": str(e),
                "function_name": function_name
            }
    
    def execute_from_tool_calls(self, tool_calls: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        """从AI模型的tool_calls执行函数"""
        results = []
        for tool_call in tool_calls:
            if tool_call.get("type") == "function":
                function_info = tool_call.get("function", {})
                function_name = function_info.get("name")
                raw_args = function_info.get("arguments", "{}")

                try:
                    if isinstance(raw_args, dict):
                        arguments = raw_args
                    elif isinstance(raw_args, str):
                        arguments = json.loads(raw_args or "{}")
                    else:
                        # 兜底：非常规类型一律包裹为 value
                        arguments = {"value": raw_args}

                    tool_logger.info(
                        f"[start] fn={function_name} tool_call_id={tool_call.get('id')} args={_preview_text(arguments)}"
                    )
                    result = self.execute(function_name, arguments)
                    result["tool_call_id"] = tool_call.get("id")
                    results.append(result)
                except Exception as e:
                    tool_logger.error(
                        f"[error] fn={function_name} tool_call_id={tool_call.get('id')} err={str(e)}"
                    )
                    results.append({
                        "success": False,
                        "error": f"参数解析错误: {e}",
                        "function_name": function_name,
                        "tool_call_id": tool_call.get("id")
                    })

        return results
    
    # ==================== 异步方法 ====================
    
    async def execute_async(self, function_name: str, arguments: Dict[str, Any]) -> Any:
        """异步执行函数调用"""
        func_schema = self.registry.get_function(function_name)
        if not func_schema:
            raise ValueError(f"未找到函数: {function_name}")
        
        try:
            # 在线程池中执行函数，避免阻塞主线程
            loop = asyncio.get_event_loop()
            
            # 创建一个包装函数来正确传递参数
            def execute_with_args():
                return func_schema.function(**arguments)
            
            logger.debug(f"[fc.async] 调用函数: {function_name} args={arguments}")
            tool_logger.info(f"[start.async] fn={function_name} args={_preview_text(arguments)}")
            result = await loop.run_in_executor(
                self.executor, 
                execute_with_args
            )
            tool_logger.info(
                f"[success.async] fn={function_name} type={type(result).__name__} preview={_preview_text(result)}"
            )
            return {
                "success": True,
                "result": result,
                "function_name": function_name
            }
        except Exception as e:
            console.print(f"[red]异步执行函数 {function_name} 时出错: {e}[/red]")
            tool_logger.error(f"[error.async] fn={function_name} err={str(e)}")
            return {
                "success": False,
                "error": str(e),
                "function_name": function_name
            }
    
    async def execute_from_tool_calls_async(self, tool_calls: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        """异步从AI模型的tool_calls执行函数"""
        tasks = []
        timeouts: List[int] = []

        for tool_call in tool_calls:
            if tool_call.get("type") == "function":
                function_info = tool_call.get("function", {})
                function_name = function_info.get("name")
                raw_args = function_info.get("arguments", "{}")

                try:
                    if isinstance(raw_args, dict):
                        arguments = raw_args
                    elif isinstance(raw_args, str):
                        arguments = json.loads(raw_args or "{}")
                    else:
                        arguments = {"value": raw_args}

                    logger.debug(f"[fc.tool] 解析参数成功: fn={function_name} args={arguments}")
                    tool_logger.info(
                        f"[start.tool] fn={function_name} tool_call_id={tool_call.get('id')} args={_preview_text(arguments)}"
                    )
                    # 创建异步任务
                    task = self._execute_single_tool_call_async(
                        function_name, arguments, tool_call.get("id")
                    )
                    tasks.append(task)
                    try:
                        timeouts.append(int(arguments.get("timeout", 30)))
                    except Exception:
                        timeouts.append(30)
                except Exception as e:
                    # 对于解析错误，创建一个返回错误的协程
                    async def create_error_result():
                        return {
                            "success": False,
                            "error": f"参数解析错误: {e}",
                            "function_name": function_name,
                            "tool_call_id": tool_call.get("id")
                        }
                    tasks.append(create_error_result())
        
        # 并发执行所有工具调用，添加超时处理
        if tasks:
            try:
                # 计算整体超时，取各工具声明的最大timeout
                overall_timeout = max(timeouts) if timeouts else 30
                logger.debug(f"[fc.tool] 并发执行工具调用，overall_timeout={overall_timeout}s, tasks={len(tasks)}")
                # 添加超时处理，避免工具执行时间过长导致卡死
                results = await asyncio.wait_for(
                    asyncio.gather(*tasks, return_exceptions=True),
                    timeout=overall_timeout
                )
                
                # 处理异常结果
                processed_results = []
                for result in results:
                    if isinstance(result, Exception):
                        processed_results.append({
                            "success": False,
                            "error": f"工具执行异常: {str(result)}",
                            "function_name": "unknown"
                        })
                        logger.error(f"[fc.tool] 工具执行异常: {str(result)} fn={function_name}")
                        tool_logger.error(f"[error.tool] fn={function_name} err={str(result)}")
                    else:
                        tool_logger.info(
                            f"[success.tool] fn={result.get('function_name')} tool_call_id={result.get('tool_call_id')} preview={_preview_text(result.get('result'))}"
                        )
                        processed_results.append(result)
                return processed_results
            except asyncio.TimeoutError:
                # 超时处理
                print("工具执行超时，强制返回")
                return [{
                    "success": False,
                    "error": f"工具执行超时，请稍后重试",
                    "function_name": "timeout"
                }]
            except Exception as e:
                # 捕获其他异常
                print(f"工具执行过程中发生异常: {e}")
                return [{
                    "success": False,
                    "error": f"工具执行过程中发生异常: {str(e)}",
                    "function_name": "error"
                }]
        
        return []
    
    async def _execute_single_tool_call_async(self, function_name: str, arguments: Dict[str, Any], tool_call_id: str) -> Dict[str, Any]:
        """异步执行单个工具调用"""
        result = await self.execute_async(function_name, arguments)
        result["tool_call_id"] = tool_call_id
        return result


# 全局函数注册表
function_registry = FunctionRegistry()
function_executor = FunctionCallExecutor(function_registry)



"""
AI大模型请求模块

提供统一的AI大模型调用接口，支持配置文件管理和回调验证。
"""

from .client import AIClient
from .config import AIConfig
from .validators import ResponseValidator
from .function_call import function_registry, function_executor, FunctionRegistry, FunctionCallExecutor

# 导入并注册所有function call函数
from .functions import register_all_functions
register_all_functions()

__all__ = ['AIClient', 'AIConfig', 'ResponseValidator', 'function_registry', 'function_executor', 'FunctionRegistry', 'FunctionCallExecutor']


"""
认证相关命令
"""
from mando import command, arg
from ketacli.sdk.base.client import do_login, do_logout
from ketacli.sdk.base.config import list_clusters


@command
def login(name="keta", endpoint="http://localhost:9000", token=""):
    """Login to ketadb, cache authentication info to ~/.keta/config.yaml

    :param repository: Repository to push to.
    :param -n, --name: The login account name. Defaults to "keta".
    :param -e, --endpoint: The ketadb endpoint. Defaults to "http://localhost:9000".
    :param -t, --token: Your keta api token, create from ketadb webui. Defaults to "".
    """
    do_login(name=name, endpoint=endpoint, token=token)


@command
@arg("name", type=str,
     completer=lambda prefix, **kwd: [x['name'] for x in list_clusters() if x['name'].startswith(prefix)])
def logout(name=None):
    """Logout from ketadb, clear authentication info

    :param -n, --name: logout from which account
    """
    do_logout(name)


"""阶段7将实现执行器与步骤循环集成；当前为占位模块。"""

from __future__ import annotations

from typing import List


def plan_from_description(user_context: dict, description: str) -> List[str]:
    """占位：后续由模型生成执行计划。"""
    return []


"""
AI响应数据验证模块

提供回调方法来验证AI大模型返回的数据格式。
"""

from typing import Any, Dict, Callable, Optional
import json
import re
from abc import ABC, abstractmethod


class BaseValidator(ABC):
    """验证器基类"""
    
    @abstractmethod
    def validate(self, response: Any) -> bool:
        """
        验证响应数据
        
        Args:
            response: 响应数据
            
        Returns:
            bool: 验证是否通过
        """
        pass
    
    @abstractmethod
    def get_error_message(self) -> str:
        """获取错误信息"""
        pass


class JSONValidator(BaseValidator):
    """JSON格式验证器"""
    
    def __init__(self, required_fields: list = None):
        """
        初始化JSON验证器
        
        Args:
            required_fields: 必需的字段列表
        """
        self.required_fields = required_fields or []
        self.error_message = ""
    
    def validate(self, response: Any) -> bool:
        """验证JSON格式和必需字段"""
        try:
            if isinstance(response, str):
                data = json.loads(response)
            elif isinstance(response, dict):
                data = response
            else:
                self.error_message = "响应数据不是有效的JSON格式"
                return False
            
            # 检查必需字段
            for field in self.required_fields:
                if field not in data:
                    self.error_message = f"缺少必需字段: {field}"
                    return False
            
            return True
        except json.JSONDecodeError as e:
            self.error_message = f"JSON解析错误: {e}"
            return False
    
    def get_error_message(self) -> str:
        return self.error_message


class RegexValidator(BaseValidator):
    """正则表达式验证器"""
    
    def __init__(self, pattern: str, flags: int = 0):
        """
        初始化正则验证器
        
        Args:
            pattern: 正则表达式模式
            flags: 正则表达式标志
        """
        self.pattern = re.compile(pattern, flags)
        self.error_message = ""
    
    def validate(self, response: Any) -> bool:
        """使用正则表达式验证响应"""
        try:
            text = str(response)
            if self.pattern.search(text):
                return True
            else:
                self.error_message = f"响应不匹配正则表达式: {self.pattern.pattern}"
                return False
        except Exception as e:
            self.error_message = f"正则验证错误: {e}"
            return False
    
    def get_error_message(self) -> str:
        return self.error_message


class LengthValidator(BaseValidator):
    """长度验证器"""
    
    def __init__(self, min_length: int = 0, max_length: int = None):
        """
        初始化长度验证器
        
        Args:
            min_length: 最小长度
            max_length: 最大长度
        """
        self.min_length = min_length
        self.max_length = max_length
        self.error_message = ""
    
    def validate(self, response: Any) -> bool:
        """验证响应长度"""
        try:
            text = str(response)
            length = len(text)
            
            if length < self.min_length:
                self.error_message = f"响应长度 {length} 小于最小长度 {self.min_length}"
                return False
            
            if self.max_length is not None and length > self.max_length:
                self.error_message = f"响应长度 {length} 大于最大长度 {self.max_length}"
                return False
            
            return True
        except Exception as e:
            self.error_message = f"长度验证错误: {e}"
            return False
    
    def get_error_message(self) -> str:
        return self.error_message


class CustomValidator(BaseValidator):
    """自定义验证器"""
    
    def __init__(self, validator_func: Callable[[Any], bool], error_message: str = "自定义验证失败"):
        """
        初始化自定义验证器
        
        Args:
            validator_func: 自定义验证函数
            error_message: 错误信息
        """
        self.validator_func = validator_func
        self.error_message = error_message
    
    def validate(self, response: Any) -> bool:
        """使用自定义函数验证响应"""
        try:
            return self.validator_func(response)
        except Exception as e:
            self.error_message = f"自定义验证错误: {e}"
            return False
    
    def get_error_message(self) -> str:
        return self.error_message


class ResponseValidator:
    """响应验证器管理器"""
    
    def __init__(self):
        self.validators = []
    
    def add_validator(self, validator: BaseValidator):
        """
        添加验证器
        
        Args:
            validator: 验证器实例
        """
        self.validators.append(validator)
    
    def add_json_validator(self, required_fields: list = None):
        """
        添加JSON验证器
        
        Args:
            required_fields: 必需的字段列表
        """
        self.add_validator(JSONValidator(required_fields))
    
    def add_regex_validator(self, pattern: str, flags: int = 0):
        """
        添加正则验证器
        
        Args:
            pattern: 正则表达式模式
            flags: 正则表达式标志
        """
        self.add_validator(RegexValidator(pattern, flags))
    
    def add_length_validator(self, min_length: int = 0, max_length: int = None):
        """
        添加长度验证器
        
        Args:
            min_length: 最小长度
            max_length: 最大长度
        """
        self.add_validator(LengthValidator(min_length, max_length))
    
    def add_custom_validator(self, validator_func: Callable[[Any], bool], error_message: str = "自定义验证失败"):
        """
        添加自定义验证器
        
        Args:
            validator_func: 自定义验证函数
            error_message: 错误信息
        """
        self.add_validator(CustomValidator(validator_func, error_message))
    
    def validate(self, response: Any) -> tuple[bool, list]:
        """
        验证响应数据
        
        Args:
            response: 响应数据
            
        Returns:
            tuple: (是否通过验证, 错误信息列表)
        """
        errors = []
        
        for validator in self.validators:
            if not validator.validate(response):
                errors.append(validator.get_error_message())
        
        return len(errors) == 0, errors
    
    def clear_validators(self):
        """清空所有验证器"""
        self.validators.clear()


# 预定义的常用验证器
def create_openai_validator() -> ResponseValidator:
    """创建OpenAI响应验证器"""
    validator = ResponseValidator()
    validator.add_json_validator(['choices'])
    return validator


def create_claude_validator() -> ResponseValidator:
    """创建Claude响应验证器"""
    validator = ResponseValidator()
    validator.add_json_validator(['content'])
    return validator


def create_text_validator(min_length: int = 1, max_length: int = None) -> ResponseValidator:
    """创建文本响应验证器"""
    validator = ResponseValidator()
    validator.add_length_validator(min_length, max_length)
    return validator


"""聊天输入组件模块"""

from textual import on
from textual.app import ComposeResult
from textual.containers import Container, Horizontal
from textual.message import Message
from textual.widgets import Button, TextArea
import logging
logger = logging.getLogger("ketacli.textual")


class CustomTextArea(TextArea):
    """自定义TextArea组件，处理Enter键发送消息"""
    
    def on_key(self, event) -> None:
        """处理键盘事件"""
        # Shift+Enter 插入换行
        if event.key == "shift+enter":
            # 让TextArea处理换行，不阻止默认行为
            return
        # 单独的Enter键发送消息
        elif event.key == "enter":
            message_text = self.text.strip()
            if message_text:  # 只有非空消息才发送
                # 发送自定义消息给父组件
                self.post_message(ChatInputWidget.MessageSent(message_text))
                # 清空输入框
                self.text = ""
            event.prevent_default()  # 阻止默认的换行行为
            event.stop()  # 停止事件传播


class ChatInputWidget(Container):
    """聊天输入组件"""
    
    class MessageSent(Message):
        """消息发送事件"""
        def __init__(self, message: str):
            super().__init__()
            self.message = message
    
    class StopRequested(Message):
        """停止请求事件"""
        pass
    class ContinueRequested(Message):
        pass
    
    # 移除 ExecuteTaskRequested 事件类
    #（已移除 ExecuteTaskRequested 事件类）
    
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._is_loading = False
        self._is_processing = False  # AI正在处理中
        self._is_paused = False
        
    def compose(self) -> ComposeResult:
        """构建输入UI"""
        with Horizontal(classes="input-row"):
            yield CustomTextArea(id="message-input", classes="message-input")
            with Horizontal(classes="input-buttons"):
                yield Button("发送", id="send-button", variant="primary")
                yield Button("清空", id="clear-button", variant="default")
                
    def on_button_pressed(self, event: Button.Pressed) -> None:
        """处理按钮点击"""
        if event.button.id == "send-button":
            if self._is_processing:
                self.post_message(self.StopRequested())
            elif self._is_paused:
                self.post_message(self.ContinueRequested())
            else:
                self._send_message()
        elif event.button.id == "clear-button":
            self._clear_input()
            

    def on_text_area_submitted(self, event) -> None:
        """处理回车发送"""
        if event.text_area.id == "message-input":
            self._send_message()
            
    def _send_message(self):
        """发送消息"""
        if self._is_loading or self._is_processing:
            return
            
        input_widget = self.query_one("#message-input", CustomTextArea)
        message = input_widget.text.strip()
        if not message:
            message = "继续"
        logger.debug(f"[input] 发送消息：len={len(message)} text_preview='{message[:120]}'")
        self.set_loading(True)
        self.post_message(self.MessageSent(message))
        input_widget.text = ""
            
    def _clear_input(self):
        """清空输入"""
        input_widget = self.query_one("#message-input", CustomTextArea)
        input_widget.text = ""
        
    def set_loading(self, loading: bool = True):
        """设置加载状态"""
        self._is_loading = loading
        send_button = self.query_one("#send-button", Button)
        input_widget = self.query_one("#message-input", CustomTextArea)
        
        if loading:
            send_button.label = "发送中..."
            send_button.disabled = True
            input_widget.disabled = True
        else:
            send_button.label = "发送"
            send_button.disabled = False
            input_widget.disabled = False
    
    def set_processing(self, processing: bool = True):
        """设置AI处理状态"""
        self._is_processing = processing
        self._is_paused = False if processing else self._is_paused
        send_button = self.query_one("#send-button", Button)
        input_widget = self.query_one("#message-input", CustomTextArea)
        
        if processing:
            send_button.label = "暂停"
            send_button.variant = "error"
            send_button.disabled = False
            input_widget.disabled = True
        else:
            send_button.label = "发送"
            send_button.variant = "primary"
            send_button.disabled = False
            input_widget.disabled = False

    def set_paused(self, paused: bool = True):
        self._is_paused = paused
        self._is_processing = False if paused else self._is_processing
        send_button = self.query_one("#send-button", Button)
        input_widget = self.query_one("#message-input", CustomTextArea)
        if paused:
            send_button.label = "继续"
            send_button.variant = "warning"
            send_button.disabled = False
            input_widget.disabled = False
        else:
            send_button.label = "发送"
            send_button.variant = "primary"
            send_button.disabled = False
            input_widget.disabled = False
